importPath=string.gsub(getfenv(1)._.Name,"%.AnimationMaint","").."."
resourcePath=string.gsub(importPath,"%.","/").."Resources/"
--[[
	-- sprites is a table of sprite definitions
	-- sprites[].Name = name of sprite - only used in AnimationMaint
	-- sprites[].CurrentImage = temporary control used to render preview in AnimationMaint, removed on exit from AnimationMaint
	-- sprites[].Width= optional width of sprite
	-- sprites[].Height= optional height of sprite
	-- sprites[].FixedSize= optional flag indicating fixed size
	-- sprites[].GetImage(self,frame) is a method that returns the image name of the sprite (either file name or resource #) for the specified frame - if nil, it is created at run-time from the FrameSets table
	-- sprites[].GetPosition(self,frame) is a method that returns the Left, Top and ZOrder for the specified frame - if nil, it is created at run-time from the FrameSets table
	-- sprites[].FrameSets = array of frame definitions
	-- sprites[].FrameSets[].ImageString = equation for Image(frame) - typically a string.format statement
	-- sprites[].FrameSets[].WidthString = equation for Width(frame) - typically blank, but could be useful in rare situations
	-- sprites[].FrameSets[].HeightString = equation for ZOrder(frame) - typically blank, but could be useful in rare situations
	-- sprites[].FrameSets[].TopString = equation for ZOrder(frame) - equation for top position per frame
	-- sprites[].FrameSets[].LeftString = equation for ZOrder(frame) - equation for left position per frame
	-- sprites[].FrameSets[].ZOrderString = equation for ZOrder(frame)
	-- sprites[].FrameSets[].StartFrame = first frame of frameset
	-- sprites[].FrameSets[].NextFrame = first frame following frameSet - is nil for last frameSet

-- allow dragging a sprite in the preview to affect the formula applied to it's left and top positions in the current frameset?
--]]
toboolean=function(val)
	local ret=false
	if type(val)=="string" then
		if val=="nil" then
			ret=nil
		elseif val~="false" then
			ret=true
		end
	elseif val then
		ret=true
	end
	return ret
end

clipboard={}
clearClipboard=function()
	if clipboard.FrameSets~=nil then
		for k,v in pairs(clipboard.FrameSets) do
			clipboard.FrameSets[k]=nil
		end
	end
	for k,v in pairs(clipboard) do
		clipboard[k]=nil
	end
end
copyFrameSetToClipboard=function(spriteIndex,frameSetIndex)
	clearClipboard()
	clipboard.Type="FrameSet"
	local source=animationMaint.TargetContent.Sprites[spriteIndex].FrameSets[frameSetIndex]
	clipboard.StartFrame=source.StartFrame -- hmmm
	clipboard.NextFrame=source.NextFrame -- hmmm
	clipboard.ImageString=source.ImageString
	clipboard.WidthString=source.WidthString
	clipboard.HeightString=source.HeightString
	clipboard.LeftString=source.LeftString
	clipboard.TopString=source.TopString
	clipboard.ZOrderString=source.ZOrderString
	clipboard.TextString=source.TextString
	clipboard.Font=source.Font
	if source.BackColor~=nil then
		clipboard.BackColor=Turbine.UI.Color(source.BackColor.R,source.BackColor.G,source.BackColor.B)
	else
		clipboard.BackColor=nil
	end
	if source.ForeColor~=nil then
		clipboard.ForeColor=Turbine.UI.Color(source.ForeColor.R,source.ForeColor.G,source.ForeColor.B)
	else
		clipboard.ForeColor=nil
	end
end
copySpriteToClipboard=function(spriteIndex)
	clearClipboard()
	clipboard.Type="Sprite"
	local source=animationMaint.TargetContent.Sprites[spriteIndex]
	clipboard.FixedSize=source.FixedSize
	clipboard.Width=source.Width
	clipboard.Height=source.Height
	clipboard.Name=source.Name
	clipboard.FrameSets={}
	for k,v in ipairs(source.FrameSets) do
		clipboard.FrameSets[k]={}
		clipboard.FrameSets[k].StartFrame=v.StartFrame
		clipboard.FrameSets[k].NextFrame=v.NextFrame
		clipboard.FrameSets[k].ImageString=v.ImageString
		clipboard.FrameSets[k].WidthString=v.WidthString
		clipboard.FrameSets[k].HeightString=v.HeightString
		clipboard.FrameSets[k].LeftString=v.LeftString
		clipboard.FrameSets[k].TopString=v.TopString
		clipboard.FrameSets[k].ZOrderString=v.ZOrderString
		clipboard.FrameSets[k].TextString=v.TextString
		clipboard.FrameSets[k].Font=v.Font
		if v.BackColor~=nil then
			clipboard.FrameSets[k].BackColor=Turbine.UI.Color(v.BackColor.R,v.BackColor.G,v.BackColor.B)
		else
			clipboard.FrameSets[k].BackColor=nil
		end
		if v.ForeColor~=nil then
			clipboard.FrameSets[k].ForeColor=Turbine.UI.Color(v.ForeColor.R,v.ForeColor.G,v.ForeColor.B)
		else
			clipboard.FrameSets[k].ForeColor=nil
		end
	end
end
pasteFrameSetFromClipboard=function(spriteIndex,frameSetIndex)
	local target=animationMaint.TargetContent.Sprites[spriteIndex].FrameSets[frameSetIndex]
	if target~=nil and clipboard.Type=="FrameSet" then
		target.ImageString=clipboard.ImageString
		target.WidthString=clipboard.WidthString
		target.HeightString=clipboard.HeightString
		target.LeftString=clipboard.LeftString
		target.TopString=clipboard.TopString
		target.ZOrderString=clipboard.ZOrderString
		target.TextString=clipboard.TextString
		target.Font=clipboard.Font
		if clipboard.BackColor~=nil then
			target.BackColor=Turbine.UI.Color(clipboard.BackColor.R,clipboard.BackColor.G,clipboard.BackColor.B)
		else
			target.BackColor=nil
		end
		if clipboard.ForeColor~=nil then
			target.ForeColor=Turbine.UI.Color(clipboard.ForeColor.R,clipboard.ForeColor.G,clipboard.ForeColor.B)
		else
			target.ForeColor=nil
		end
	end
end
pasteSpriteFromClipboard=function(spriteIndex)
	if spriteIndex==nil then spriteIndex=animationMaint.CurrentSpriteIndex end
	local target=animationMaint.TargetContent.Sprites[spriteIndex]
	if target~=nil and clipboard.Type=="Sprite" then
		target.FixedSize=clipboard.FixedSize
		target.Width=clipboard.Width
		target.Height=clipboard.Height
		target.Name=clipboard.Name
		if target.Name=="" then
			spriteControls[spriteIndex].Header.Name:SetText(spriteIndex)
		else
			spriteControls[spriteIndex].Header.Name:SetText(target.Name)
		end
		-- clear all existing frameset controls
		for k=1,#target.FrameSets do
			removeFrameSetControl(spriteIndex,1)
		end
		target.FrameSets={}
		for k,v in ipairs(clipboard.FrameSets) do
			target.FrameSets[k]={}
			target.FrameSets[k].StartFrame=v.StartFrame
			target.FrameSets[k].NextFrame=v.NextFrame
			target.FrameSets[k].ImageString=v.ImageString
			target.FrameSets[k].WidthString=v.WidthString
			target.FrameSets[k].HeightString=v.HeightString
			target.FrameSets[k].LeftString=v.LeftString
			target.FrameSets[k].TopString=v.TopString
			target.FrameSets[k].ZOrderString=v.ZOrderString
			target.FrameSets[k].TextString=v.TextString
			target.FrameSets[k].Font=v.Font
			if v.BackColor~=nil then
				target.FrameSets[k].BackColor=Turbine.UI.Color(v.BackColor.R,v.BackColor.G,v.BackColor.B)
			else
				target.FrameSets[k].BackColor=nil
			end
			if v.ForeColor~=nil then
				target.FrameSets[k].ForeColor=Turbine.UI.Color(v.ForeColor.R,v.ForeColor.G,v.ForeColor.B)
			else
				target.FrameSets[k].ForeColor=nil
			end
			newFrameSetControl(spriteControls[spriteIndex],v.StartFrame,v.NextFrame,k)
		end
	end
end
spriteControls={}
newFrameSetControl=function(spriteControl,startFrame,nextFrame,newFrameSetIndex)
	if newFrameSetIndex==nil then
		-- inserting, need to determine newFrameSetIndex and nextFrame from existing framesets based on startFrame
		for k,v in ipairs(spriteControl.List.FrameSets) do
			if v.StartFrame<startFrame and (v.NextFrame==nil or v.NextFrame>startFrame) then
				newFrameSetIndex=k+1
				nextFrame=v.NextFrame
				v.NextFrame=startFrame
				-- adjust existing control width
				local newWidth=(v.NextFrame-v.StartFrame)*3
				v:SetWidth(newWidth)
				v.EndMarker:SetLeft(newWidth-2)
			elseif v.StartFrame>startFrame then
				v.Index=v.Index+1
			end
		end
	end
	local left=startFrame*3-3
	local width
	if nextFrame==nil then
		width=animationMaint.TimelineList:GetWidth()-left
	else
		width=nextFrame*3-3-left
	end
	local newFrameSet=Turbine.UI.Control()
	newFrameSet:SetParent(spriteControl.List)
	newFrameSet.Index=newFrameSetIndex
	newFrameSet.StartFrame=startFrame
	newFrameSet.NextFrame=nextFrame
	newFrameSet:SetSize(width,animationMaint.TimelineRowHeight)
	newFrameSet:SetLeft(left)
	newFrameSet:SetBackColor(animationMaint.FrameSetColor)
	newFrameSet:SetHeight(animationMaint.TimelineRowHeight)
	newFrameSet.MouseClick=function(sender,args)
		local parent=sender:GetParent()
		if animationMaint.SelectedFrameSet~=nil then
			animationMaint.SelectedFrameSet:SetBackColor(animationMaint.FrameSetColor)
		end
		animationMaint.SelectedFrameSet=sender
		sender:SetBackColor(animationMaint.SelectedFrameSetColor)
		refreshFrameSetList(parent.SpriteIndex,sender.Index)
		refreshCurrFrameList()
		if args.Button==2 then
			local frame=(math.floor(args.X/3)+sender.StartFrame)
			frameSetMenu:GetItems():Get(1).Click=function()
				timelineSplitFrameSet(parent.SpriteIndex,sender.Index,frame)
			end
			frameSetMenu:GetItems():Get(3).Click=function()
				timelineDeleteFrameSet(parent.SpriteIndex,sender.Index)
			end
			frameSetMenu:GetItems():Get(4).Click=function()
				timelineDeleteSprite(parent.SpriteIndex)
			end
			frameSetMenu:GetItems():Get(7):SetEnabled(clipboard.Type=="FrameSet")
			frameSetMenu:GetItems():Get(8):SetEnabled(clipboard.Type=="Sprite")
			frameSetMenu:ShowMenu()
		end
	end
	newFrameSet.MouseEnter=function(sender,args)
		x,y=sender:GetMousePosition()
		if x<3 then
			if sender.StartFrame~=1 then
				animationMaint.Message:SetText(Resource[language][284])
			end
		elseif x>sender:GetWidth()-3 then
			if sender.NextFrame~=nil then
				animationMaint.Message:SetText(Resource[language][284])
			end
		else
			animationMaint.Message:SetText(Resource[language][285])
		end
	end
	newFrameSet.MouseLeave=function()
		animationMaint.Message:SetText("")
	end
	newFrameSet.MouseMove=function(sender,args)
		animationMaint.Message:SetText(Resource[language][285])
	end
	newFrameSet.StartMarker=Turbine.UI.Control()
	newFrameSet.StartMarker:SetParent(newFrameSet)
	newFrameSet.StartMarker:SetSize(2,animationMaint.TimelineRowHeight)
	newFrameSet.StartMarker:SetBackColor(Turbine.UI.Color.Green)
	newFrameSet.StartMarker.MouseClick=function(sender,args)
		-- pass MouseClick through to parent, we only want MouseDown, MouseUp and MouseMove
		args.X=args.X+sender:GetLeft()
		local parent=sender:GetParent()
		parent.MouseClick(parent,Table.Copy(args))
	end
	newFrameSet.StartMarker.MouseDown=function(sender,args)
		local parent=sender:GetParent()
		if parent.StartFrame~=1 then
			-- this IS the NextFrameCtrl
			sender.CurrentFrameCtrl=parent:GetParent().FrameSets[parent.Index-1]
			sender.NextFrameCtrl=parent
			sender.Dragging=true
			sender.X=args.X
		end
	end
	newFrameSet.StartMarker.MouseMove=function(sender,args)
		if sender.Dragging then
			frameSetDrag(sender.CurrentFrameCtrl,sender.NextFrameCtrl,sender,args.X)
		elseif sender:GetParent().StartFrame~=1 then
			animationMaint.Message:SetText(Resource[language][284])
		end
	end
	newFrameSet.StartMarker.MouseUp=function(sender,args)
		sender.Dragging=false
	end
	newFrameSet.EndMarker=Turbine.UI.Control()
	newFrameSet.EndMarker:SetParent(newFrameSet)
	newFrameSet.EndMarker:SetSize(2,animationMaint.TimelineRowHeight)
	newFrameSet.EndMarker:SetLeft(width-2)
	newFrameSet.EndMarker:SetBackColor(Turbine.UI.Color.Red)
	newFrameSet.EndMarker.MouseClick=function(sender,args)
		-- pass MouseClick through to parent, we only want MouseDown, MouseUp and MouseMove
		args.X=args.X+sender:GetLeft()
		local parent=sender:GetParent()
		parent.MouseClick(parent,Table.Copy(args))
	end
	newFrameSet.EndMarker.MouseDown=function(sender,args)
		local parent=sender:GetParent()
		if parent.NextFrame~=nil then
			-- this IS the NextFrameCtrl
			sender.CurrentFrameCtrl=parent
			sender.NextFrameCtrl=parent:GetParent().FrameSets[parent.Index+1]
			sender.Dragging=true
			sender.X=args.X
		end
	end
	newFrameSet.EndMarker.MouseMove=function(sender,args)
		if sender.Dragging then
			frameSetDrag(sender.CurrentFrameCtrl,sender.NextFrameCtrl,sender,args.X)
		elseif sender:GetParent().NextFrame~=nil then
			animationMaint.Message:SetText(Resource[language][284])
		end
	end
	newFrameSet.EndMarker.MouseUp=function(sender,args)
		sender.Dragging=false
	end
	table.insert(spriteControl.List.FrameSets,newFrameSetIndex,newFrameSet)
	return
end
newSpriteControl=function(newIndex)
	if newIndex==nil then newIndex=#spriteControls+1 end
	local newControl={}
	newControl.CurrentImage=Turbine.UI.Label()
	newControl.CurrentImage:SetParent(animationMaint.PreviewBackground)
	newControl.CurrentImage:SetBlendMode(Turbine.UI.BlendMode.Overlay)
	newControl.CurrentImage:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	newControl.Header=Turbine.UI.Control()
	newControl.Header.SpriteIndex=newIndex
	newControl.Header:SetParent(animationMaint.TimelineHeader)
	newControl.Header:SetSize(animationMaint.TimelineHeader:GetWidth(),animationMaint.TimelineRowHeight)
	newControl.Header:SetBackColor(Settings.BackColor)
	newControl.Header:SetTop(animationMaint.TimelineHeader:GetHeight()+1)
	animationMaint.TimelineHeader:SetHeight(animationMaint.TimelineHeader:GetHeight()+animationMaint.TimelineRowHeight+1)
	newControl.Header.Name=Turbine.UI.TextBox()
	newControl.Header.Name:SetParent(newControl.Header)
	newControl.Header.Name:SetSize(animationMaint.TimelineHeader:GetWidth()-20,animationMaint.TimelineRowHeight)
	newControl.Header.Name:SetLeft(20)
	newControl.Header.Name:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
	newControl.Header.Name:SetFont(Settings.Font)
	newControl.Header.Name:SetMultiline(false)
	if animationMaint.TargetContent.Sprites[newIndex].Name==nil then
		newControl.Header.Name:SetText(tostring(newIndex))
	else
		newControl.Header.Name:SetText(animationMaint.TargetContent.Sprites[newIndex].Name)
	end
	newControl.Header.Name.MouseClick=function(sender,args)
		-- send click to first frameset of this sprite
		local spriteIndex=sender:GetParent().SpriteIndex
		local target=spriteControls[spriteIndex].List.FrameSets[1]
		target.MouseClick(target,{["Button"]=1,X=0,Y=0})
		refreshFrameSetList(sender:GetParent().SpriteIndex,1)
		refreshCurrFrameList()
	end
	newControl.Header.Name.KeyDown=function(sender,args)
		if args.Action==75 then
			sender:TextChanged()
		end
	end
	newControl.Header.Name.TextChanged=function(sender,args)
		local name=string.trim(sender:GetText())
		local parent=sender:GetParent()
		if name=="" then
			name=parent.SpriteIndex
		end
		animationMaint.TargetContent.Sprites[parent.SpriteIndex].Name=name
		-- need to update in FrameSetList and CurrFrameList since clicking on this control set this sprite as the current sprite
		refreshFrameSetList(parent.SpriteIndex)
		refreshCurrFrameList()
		authorWindow.SetChangedState(true)
	end


	newControl.Header.CheckBox=Turbine.UI.Lotro.CheckBox()
	newControl.Header.CheckBox:SetParent(newControl.Header)
	newControl.Header.CheckBox:SetSize(20,20)
	newControl.Header.CheckBox:SetTop(animationMaint.TimelineRowHeight/2-10)
	newControl.Header.CheckBox:SetChecked(true)
	newControl.Header.CheckBox:SetText(" ")
	newControl.Header.CheckBox.CheckedChanged=function(sender,args)
		refreshAnimationPreview() -- can't just make it visible incase the defined image is nil
	end
	newControl.List=Turbine.UI.Control()
	newControl.List.SpriteIndex=newIndex
	newControl.List:SetParent(animationMaint.TimelineList)
	newControl.List:SetSize(animationMaint.TimelineList:GetWidth(),animationMaint.TimelineRowHeight)
	newControl.List:SetTop(newControl.Header:GetTop())
	animationMaint.TimelineList:SetHeight(animationMaint.TimelineHeader:GetHeight())
	newControl.List.FrameSets={}
	for k,v in ipairs(animationMaint.TargetContent.Sprites[newIndex].FrameSets) do
		newFrameSetControl(newControl,v.StartFrame,v.NextFrame,k)
	end
	table.insert(spriteControls,newIndex,newControl)
	return
end
removeFrameSetControl=function(spriteIndex,framesetIndex)
	local list=spriteControls[spriteIndex].List
	local frameset=list.FrameSets[framesetIndex]
	frameset:SetVisible(false)
	frameset:SetParent(nil)
	frameset.StartMarker:SetVisible(false)
	frameset.StartMarker:SetParent(nil)
	frameset.StartMarker=nil
	frameset.EndMarker:SetVisible(false)
	frameset.EndMarker:SetParent(nil)
	frameset.EndMarker=nil
	--when removing, the prior frameset expands to include this frameset unless it is the first frameset in which case the second frameset becomes the first frameset
	if framesetIndex==1 then
		-- if frameset exists, it expands and moves left
		local next=list.FrameSets[2]
		if next~=nil then
			next.StartFrame=1
			next:SetLeft(0)
			next:SetWidth(next:GetWidth()+frameset:GetWidth())
			next.EndMarker:SetLeft(next:GetWidth()-2)
			
		end
	else
		-- prior frameset expands
		local prior=list.FrameSets[framesetIndex-1]
		prior.NextFrame=frameset.NextFrame
		prior:SetWidth(prior:GetWidth()+frameset:GetWidth())
		prior.EndMarker:SetLeft(prior:GetWidth()-2)
	end
	table.remove(list.FrameSets,framesetIndex)
	for k,v in pairs(list.FrameSets) do
		if k>=framesetIndex then
			v.Index=k
		end
	end
	frameset=nil
	animationMaint.CurrentFrameSetIndex=nil
end
removeSpriteControl=function(spriteIndex)
	local sprite=spriteControls[spriteIndex]
	if sprite.CurrentImage~=nil then
		sprite.CurrentImage:SetVisible(false)
		sprite.CurrentImage:SetParent(nil)
		sprite.CurrentImage=nil
	end
	sprite.Header:SetVisible(false)
	sprite.Header:SetParent(nil)
	sprite.Header.Name:SetVisible(false)
	sprite.Header.Name:SetParent(nil)
	sprite.Header.Name=nil
	sprite.Header.CheckBox:SetVisible(false)
	sprite.Header.CheckBox:SetParent(nil)
	sprite.Header.CheckBox=nil
	sprite.Header=nil
	sprite.List:SetVisible(false)
	sprite.List:SetParent(false)
	if sprite.List.FrameSets~=nil then
		for i=1,#sprite.List.FrameSets do
			removeFrameSetControl(spriteIndex,1) -- keep removing first entry since they move left to fill gap
		end
	end
	sprite.List.FrameSets=nil
	sprite.List=nil
	table.remove(spriteControls,spriteIndex)
	for k,v in ipairs(spriteControls) do
		if k>=spriteIndex then
			local newTop=v.Header:GetTop()-animationMaint.TimelineRowHeight-1
			v.Header:SetTop(newTop)
			v.List:SetTop(newTop)
			v.Header.SpriteIndex=k
			v.List.SpriteIndex=k
		end
	end
	animationMaint.TimelineHeader:SetHeight(animationMaint.TimelineHeader:GetHeight()-animationMaint.TimelineRowHeight-1)
	animationMaint.TimelineList:SetHeight(animationMaint.TimelineHeader:GetHeight())
	animationMaint.CurrentSpriteIndex=nil
end
clearSpriteControls=function()
	-- clear spriteControls
	for spriteIndex=1,#spriteControls do
		removeSpriteControl(1) -- keep removing first entry since they move down to fill gap
	end
	spriteControls={}
	animationMaint.TimelineHeader:SetHeight(0)
	animationMaint.TimelineList:SetHeight(0)
end
if AuthorSettings==nil then AuthorSettings={} end
if AuthorSettings.AnimationMaint==nil then AuthorSettings.AnimationMaint={} end
if AuthorSettings.AnimationMaint.Split1==nil then
	AuthorSettings.AnimationMaint.Split1=.5
end
if AuthorSettings.AnimationMaint.Split2==nil then
	AuthorSettings.AnimationMaint.Split2=.3333
end
if AuthorSettings.AnimationMaint.Split3==nil then
	AuthorSettings.AnimationMaint.Split3=.6667
end
if AuthorSettings.AnimationMaint.Split3<AuthorSettings.AnimationMaint.Split2 then AuthorSettings.AnimationMaint.Split3=AuthorSettings.AnimationMaint.Split2+1/(displayWidth-2) end

frameHover=Turbine.UI.ContextMenu();
frameHover:GetItems():Add(Turbine.UI.MenuItem(""));
frameHover:GetItems():Get(1).Click=function(sender,args)
	local num=tonumber(sender:GetText())
	if num==nil or num<1 then num=1 end
	animationMaint.FrameScroll:SetValue(num)
end
frameSetMenu=Turbine.UI.ContextMenu()
frameSetMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][282]))
frameSetMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][283]));
frameSetMenu:GetItems():Get(2).Click=function(sender,args)
	addSprite()
end
frameSetMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][291]))
frameSetMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][292]))
frameSetMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][287])); -- copy frameset
frameSetMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][286])); -- copy sprite
frameSetMenu:GetItems():Get(5).Click=function(sender,args)
	if animationMaint.CurrentSpriteIndex~=nil and animationMaint.CurrentFrameSetIndex~=nil then
		copyFrameSetToClipboard(animationMaint.CurrentSpriteIndex,animationMaint.CurrentFrameSetIndex)
	end
end
frameSetMenu:GetItems():Get(6).Click=function(sender,args)
	if animationMaint.CurrentSpriteIndex~=nil then
		copySpriteToClipboard(animationMaint.CurrentSpriteIndex)
	end
end
frameSetMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][289])); -- paste frameset
frameSetMenu:GetItems():Get(7).Click=function(sender,args)
	if animationMaint.CurrentSpriteIndex~=nil and animationMaint.CurrentFrameSetIndex~=nil and clipboard.Type=="FrameSet" then
		pasteFrameSetFromClipboard(animationMaint.CurrentSpriteIndex,animationMaint.CurrentFrameSetIndex)
		refreshFrameSetList()
		refreshCurrFrameList()
	end
end
frameSetMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][288])); -- paste sprite
frameSetMenu:GetItems():Get(8).Click=function(sender,args)
	if animationMaint.CurrentSpriteIndex~=nil and clipboard.Type=="Sprite" then
		pasteSpriteFromClipboard(animationMaint.CurrentSpriteIndex)
		refreshFrameSetList()
		refreshCurrFrameList()
	end
end

animationMaint=Turbine.UI.Window()
animationMaint.FrameSetColor=Turbine.UI.Color(.2,.2,1)
animationMaint.SelectedFrameSetColor=Turbine.UI.Color(0,0,.8)

animationMaint:SetSize(displayWidth,displayHeight)
animationMaint.Border=Turbine.UI.Control()
animationMaint.Border:SetParent(animationMaint)
animationMaint.Border:SetPosition(1,1)

animationMaint.HeaderPanel=Turbine.UI.Control()
animationMaint.HeaderPanel:SetParent(animationMaint.Border)
-- need fields for title, width, height, backimage, backcolor, fps, path
animationMaint.Title=Turbine.UI.Label()
animationMaint.Title:SetParent(animationMaint.HeaderPanel)
animationMaint.Title:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
animationMaint.Title:SetMultiline(false)
animationMaint.WidthCaption=Turbine.UI.Label()
animationMaint.WidthCaption:SetParent(animationMaint.HeaderPanel)
animationMaint.Width=Turbine.UI.Lotro.TextBox()
animationMaint.Width:SetParent(animationMaint.HeaderPanel)
animationMaint.Width:SetMultiline(false)
animationMaint.Width.KeyDown=function(sender,args)
	if args.Action==75 then -- del key
		animationMaint.Width.TextChanged()
	end
	if args.Action==162 then -- enter key
		SettingQueueCallback(animationMaint.Height.Focus,animationMaint.Height)
	end
end
animationMaint.Width.FocusLost=function()
	refreshAnimationPreview()
end
animationMaint.Width.TextChanged=function()
	local val=tonumber(animationMaint.Width:GetText())
	animationMaint.TargetContent.Width=val
	if not animationMaint.loading then authorWindow.SetChangedState(true) end
	refreshAnimationPreview()
end
animationMaint.HeightCaption=Turbine.UI.Label()
animationMaint.HeightCaption:SetParent(animationMaint.HeaderPanel)
animationMaint.Height=Turbine.UI.Lotro.TextBox()
animationMaint.Height:SetParent(animationMaint.HeaderPanel)
animationMaint.Height:SetMultiline(false)
animationMaint.Height.KeyDown=function(sender,args)
	if args.Action==75 then
		animationMaint.Height.TextChanged()
	end
	if args.Action==162 then -- enter key
		SettingQueueCallback(animationMaint.FPS.Focus,animationMaint.FPS)
	end
end
animationMaint.Height.FocusLost=function()
	refreshAnimationPreview()
end
animationMaint.Height.TextChanged=function()
	local val=tonumber(animationMaint.Height:GetText())
	animationMaint.TargetContent.Height=val
	if not animationMaint.loading then authorWindow.SetChangedState(true) end
	refreshAnimationPreview()
end
animationMaint.FPSCaption=Turbine.UI.Label()
animationMaint.FPSCaption:SetParent(animationMaint.HeaderPanel)
animationMaint.FPS=Turbine.UI.Lotro.TextBox()
animationMaint.FPS:SetParent(animationMaint.HeaderPanel)
animationMaint.FPS:SetMultiline(false)
animationMaint.FPS.KeyDown=function(sender,args)
	if args.Action==75 then
		animationMaint.FPS.TextChanged()
	end
	if args.Action==162 then -- enter key
		SettingQueueCallback(animationMaint.FrameCount.Focus,animationMaint.FrameCount)
	end
end
animationMaint.FPS.FocusLost=function()
	refreshAnimationPreview()
end
animationMaint.FPS.TextChanged=function()
	local val=tonumber(animationMaint.FPS:GetText())
	animationMaint.TargetContent.FPS=val
	if not animationMaint.loading then authorWindow.SetChangedState(true) end
end

animationMaint.FrameCountCaption=Turbine.UI.Label()
animationMaint.FrameCountCaption:SetParent(animationMaint.HeaderPanel)
animationMaint.FrameCount=Turbine.UI.Lotro.TextBox()
animationMaint.FrameCount:SetParent(animationMaint.HeaderPanel)
animationMaint.FrameCount:SetMultiline(false)
animationMaint.FrameCount.KeyDown=function(sender,args)
	if args.Action==75 then
		animationMaint.FrameCount.TextChanged()
	end
	if args.Action==162 then -- enter key
		animationMaint.FrameCount.FocusLost()
	end
end
animationMaint.FrameCount.FocusLost=function()
	refreshAnimationPreview()
end
animationMaint.FrameCount.TextChanged=function()
	local count=tonumber(animationMaint.FrameCount:GetText())
	if count~=nil then
		if count<1 then count=1 end -- need to set minimum of 1, otherwise the scrollbar minimum gets messed up
		if animationMaint.FrameScroll:GetValue()>count then
			animationMaint.FrameScroll:SetValue(count)
		end
		animationMaint.FrameScroll:SetMaximum(count)
	end
	animationMaint.TargetContent.FrameCount=count
	if not animationMaint.loading then authorWindow.SetChangedState(true) end
	refreshTimeline()
end

animationMaint.ImagePathCaption=Turbine.UI.Label()
animationMaint.ImagePathCaption:SetParent(animationMaint.HeaderPanel)
animationMaint.ImagePath=Turbine.UI.Lotro.TextBox()
animationMaint.ImagePath:SetParent(animationMaint.HeaderPanel)
animationMaint.ImagePath:SetMultiline(false)
animationMaint.ImagePath.KeyDown=function(sender,args)
	if args.Action==75 then
		animationMaint.ImagePath.TextChanged()
	end
	if args.Action==162 then -- enter key
		SettingQueueCallback(animationMaint.BackImage.Focus,animationMaint.BackImage)
	end
end
animationMaint.ImagePath.FocusLost=function()
	refreshAnimationPreview()
end
animationMaint.ImagePath.TextChanged=function()
	animationMaint.TargetContent.Path=animationMaint.ImagePath:GetText()
	if not animationMaint.loading then authorWindow.SetChangedState(true) end
	refreshAnimationPreview()
end
animationMaint.BackImageCaption=Turbine.UI.Label()
animationMaint.BackImageCaption:SetParent(animationMaint.HeaderPanel)
animationMaint.BackImage=Turbine.UI.Lotro.TextBox()
animationMaint.BackImage:SetParent(animationMaint.HeaderPanel)
animationMaint.BackImage:SetMultiline(false)
animationMaint.BackImage.KeyDown=function(sender,args)
	if args.Action==75 then
		animationMaint.BackImage.TextChanged()
	end
	if args.Action==162 then -- enter key
		SettingQueueCallback(animationMaint.UseBackColor.Focus,animationMaint.UseBackColor)
	end
end
animationMaint.BackImage.FocusLost=function()
	refreshAnimationPreview()
end
animationMaint.BackImage.TextChanged=function()
	local val=animationMaint.BackImage:GetText()
	if tonumber(val)~=nil and tonumber(val)>0 then
		animationMaint.TargetContent.FileName=tonumber(val)
	else
		animationMaint.TargetContent.FileName=val
	end
	if not animationMaint.loading then authorWindow.SetChangedState(true) end
	refreshAnimationPreview()
end
animationMaint.BackColorCaption=Turbine.UI.Label()
animationMaint.BackColorCaption:SetParent(animationMaint.HeaderPanel)
animationMaint.UseBackColor=Turbine.UI.Lotro.CheckBox()
animationMaint.UseBackColor:SetParent(animationMaint.HeaderPanel)
animationMaint.UseBackColor.CheckedChanged=function()
	animationMaint.BackColor:SetVisible(animationMaint.UseBackColor:IsChecked())
	local val=animationMaint.BackColor:GetColor()
	if animationMaint.UseBackColor:IsChecked() then
		animationMaint.PreviewBackground:SetBackColor(animationMaint.BackColor:GetColor())
	else
		val=nil
		animationMaint.PreviewBackground:SetBackColor(nil)
	end
	animationMaint.TargetContent.BackColor=val
	if not animationMaint.loading then authorWindow.SetChangedState(true) end
	refreshAnimationPreview()
end
animationMaint.BackColor=ColorPicker()
animationMaint.BackColor:SetParent(animationMaint.HeaderPanel)
animationMaint.BackColor:SetColor(Settings.BackColor)
animationMaint.BackColor:SetVisible(false)
animationMaint.BackColor.ColorChanged=function()
	local val=animationMaint.BackColor:GetColor()
	if not animationMaint.UseBackColor:IsChecked() then
		val=nil
	end
	animationMaint.TargetContent.BackColor=val
	if not animationMaint.loading then authorWindow.SetChangedState(true) end
	refreshAnimationPreview()
end

-- create seperators
animationMaint.Sep1=Turbine.UI.Control()
animationMaint.Sep1:SetParent(animationMaint.Border)
animationMaint.Sep1:SetHeight(3)
animationMaint.Sep2=Turbine.UI.Control()
animationMaint.Sep2:SetParent(animationMaint.Border)
animationMaint.Sep2:SetWidth(3)
animationMaint.Sep3=Turbine.UI.Control()
animationMaint.Sep3:SetParent(animationMaint.Border)
animationMaint.Sep3:SetWidth(3)

animationMaint.Sep1.MouseDown=function(sender,args)
	animationMaint.MovingSep1=true
	animationMaint.Sep1Y=args.Y
end
animationMaint.Sep1.MouseEnter=function()
	animationMaint.Message:SetText(Resource[language][268])
end
animationMaint.Sep1.MouseLeave=function()
	animationMaint.Message:SetText("")
end
animationMaint.Sep1.MouseUp=function(sender,args)
	animationMaint.MovingSep1=false
end
animationMaint.Sep1.MouseMove=function(sender,args)
	if animationMaint.MovingSep1 then
		AuthorSettings.AnimationMaint.Split1=(animationMaint.Sep1:GetTop()+args.Y-animationMaint.Sep1Y)/animationMaint.Border:GetHeight()
		if AuthorSettings.AnimationMaint.Split1>.9 then AuthorSettings.AnimationMaint.Split1=.9 end
		if AuthorSettings.AnimationMaint.Split1<.2 then AuthorSettings.AnimationMaint.Split1=.2 end
		animationMaint.RefreshLayout()
	end
end

animationMaint.Sep2.MouseDown=function(sender,args)
	animationMaint.MovingSep2=true
	animationMaint.Sep2X=args.X
end
animationMaint.Sep2.MouseEnter=function()
	animationMaint.Message:SetText(Resource[language][268])
end
animationMaint.Sep2.MouseLeave=function()
	animationMaint.Message:SetText("")
end
animationMaint.Sep2.MouseUp=function(sender,args)
	animationMaint.MovingSep2=false
end
animationMaint.Sep2.MouseMove=function(sender,args)
	if animationMaint.MovingSep2 then
		AuthorSettings.AnimationMaint.Split2=(animationMaint.Sep2:GetLeft()+args.X-animationMaint.Sep2X)/animationMaint.Border:GetWidth()
		if AuthorSettings.AnimationMaint.Split2>AuthorSettings.AnimationMaint.Split3-6/animationMaint.Border:GetWidth() then AuthorSettings.AnimationMaint.Split2=AuthorSettings.AnimationMaint.Split3-6/animationMaint.Border:GetWidth() end
		if AuthorSettings.AnimationMaint.Split2<.1 then AuthorSettings.AnimationMaint.Split2=.1 end
		animationMaint.RefreshLayout()
	end
end

animationMaint.Sep3.MouseDown=function(sender,args)
	animationMaint.MovingSep3=true
	animationMaint.Sep3X=args.X
end
animationMaint.Sep3.MouseEnter=function()
	animationMaint.Message:SetText(Resource[language][268])
end
animationMaint.Sep3.MouseLeave=function()
	animationMaint.Message:SetText("")
end
animationMaint.Sep3.MouseUp=function(sender,args)
	animationMaint.MovingSep3=false
end
animationMaint.Sep3.MouseMove=function(sender,args)
	if animationMaint.MovingSep3 then
		AuthorSettings.AnimationMaint.Split3=(animationMaint.Sep3:GetLeft()+args.X-animationMaint.Sep3X)/animationMaint.Border:GetWidth()
		if AuthorSettings.AnimationMaint.Split3<AuthorSettings.AnimationMaint.Split2+6/animationMaint.Border:GetWidth() then AuthorSettings.AnimationMaint.Split3=AuthorSettings.AnimationMaint.Split2+6/animationMaint.Border:GetWidth() end
		if AuthorSettings.AnimationMaint.Split3>.9 then AuthorSettings.AnimationMaint.Split3=.9 end
		animationMaint.RefreshLayout()
	end
end
animationMaint.FrameSetPanel=Turbine.UI.Control()
animationMaint.FrameSetPanel:SetParent(animationMaint.Border)
animationMaint.FrameSetList=Turbine.UI.ListBox()
animationMaint.FrameSetList:SetParent(animationMaint.FrameSetPanel)
animationMaint.FrameSetListVScroll=Turbine.UI.Lotro.ScrollBar()
animationMaint.FrameSetListVScroll:SetParent(animationMaint.FrameSetPanel)
animationMaint.FrameSetListVScroll:SetOrientation(Turbine.UI.Orientation.Vertical)
animationMaint.FrameSetListVScroll:SetWidth(10)
animationMaint.FrameSetList:SetVerticalScrollBar(animationMaint.FrameSetListVScroll)

animationMaint.PreviewPanel=Turbine.UI.Control()
animationMaint.PreviewPanel:SetParent(animationMaint.Border)
animationMaint.PreviewBackground=Turbine.UI.Control()
animationMaint.PreviewBackground:SetParent(animationMaint.PreviewPanel)
animationMaint.PreviewBackground:SetBlendMode(Turbine.UI.BlendMode.Overlay)

animationMaint.CurrFramePanel=Turbine.UI.Control()
animationMaint.CurrFramePanel:SetParent(animationMaint.Border)
animationMaint.CurrFrameList=Turbine.UI.ListBox()
animationMaint.CurrFrameList:SetParent(animationMaint.CurrFramePanel)
animationMaint.CurrFrameListVScroll=Turbine.UI.Lotro.ScrollBar()
animationMaint.CurrFrameListVScroll:SetParent(animationMaint.CurrFramePanel)
animationMaint.CurrFrameListVScroll:SetOrientation(Turbine.UI.Orientation.Vertical)
animationMaint.CurrFrameListVScroll:SetWidth(10)
animationMaint.CurrFrameList:SetVerticalScrollBar(animationMaint.CurrFrameListVScroll)

animationMaint.FrameScroll=Turbine.UI.Lotro.ScrollBar()
animationMaint.FrameScroll:SetParent(animationMaint.Border)
animationMaint.FrameScroll:SetOrientation(Turbine.UI.Orientation.Horizontal)
animationMaint.FrameScroll:SetValue(1)
animationMaint.FrameScroll:SetMinimum(1)
animationMaint.FrameScroll:SetMaximum(1)
animationMaint.FrameScroll:SetHeight(10)
animationMaint.FrameScroll.ValueChanged=function()
	local frame=animationMaint.FrameScroll:GetValue()
	refreshAnimationPreview(frame)
	refreshCurrFrameList(frame)
	local left=animationMaint.TimelineHeader:GetWidth()+frame*3-animationMaint.TimelineHScroll:GetValue()-3
	animationMaint.TimelineFrameMarker:SetVisible(left>=animationMaint.TimelineHeader:GetWidth())
	animationMaint.TimelineFrameMarker:SetLeft(left)
end

animationMaint.TimelinePanel=Turbine.UI.Control()
animationMaint.TimelinePanel:SetParent(animationMaint.Border)
animationMaint.TimelinePanel:SetLeft(1)
animationMaint.TimelineRowHeight=Turbine.UI.Lotro.FontInfo[Settings.Font].size*2+4
animationMaint.TimelineHeaderCaption=Turbine.UI.Label()
animationMaint.TimelineHeaderCaption:SetParent(animationMaint.TimelinePanel)
animationMaint.TimelineHeaderCaption:SetSize(200,animationMaint.TimelineRowHeight)
animationMaint.TimelineHeaderCaption:SetFont(Settings.Font)
animationMaint.TimelineHeaderCaption:SetText(Resource[language][269])
animationMaint.TimelineHeaderCaption:SetBackColor(Settings.BackColor)
animationMaint.TimelineHeaderCaption:SetZOrder(4)

animationMaint.TimelineHeader=Turbine.UI.Control() -- non-horizontally-scrolling header column
animationMaint.TimelineHeader:SetParent(animationMaint.TimelinePanel)
animationMaint.TimelineHeader:SetZOrder(3) -- above list AND marker
animationMaint.TimelineHeader:SetBackColor(Settings.BackColor)
animationMaint.TimelineHeader:SetWidth(200)
-- add caption row
animationMaint.TimelineMeasure=Turbine.UI.Control()
animationMaint.TimelineMeasure:SetParent(animationMaint.TimelinePanel)
animationMaint.TimelineMeasure:SetHeight(animationMaint.TimelineRowHeight)
animationMaint.TimelineMeasure:SetBlendMode(Turbine.UI.BlendMode.Overlay)
animationMaint.TimelineMeasure:SetBackground(resourcePath.."Timeline.tga") -- let it tile to automatically cover the whole timeline
animationMaint.TimelineMeasure:SetZOrder(1) -- above list but below marker and caption
animationMaint.TimelineMeasure.MouseHover=function(sender,args)
	frameHover:GetItems():Get(1):SetText(math.floor(args.X/3)+1)
	local x,y=Turbine.UI.Display:GetMousePosition()
	frameHover:ShowMenuAt(x,animationMaint.TimelinePanel:GetTop()+animationMaint.TimelineMeasure:GetTop()+animationMaint.Border:GetTop()-10)
end
animationMaint.TimelineMeasure.MouseClick=function(sender,args)
	animationMaint.FrameScroll:SetValue(math.floor(args.X/3)+1)
end
animationMaint.TimelineList=Turbine.UI.Control() -- doesn't use a ListBox because of non-scrolling header and overlaid frame marker
animationMaint.TimelineList:SetParent(animationMaint.TimelinePanel)
animationMaint.TimelineVScroll=Turbine.UI.Lotro.ScrollBar()
animationMaint.TimelineVScroll:SetParent(animationMaint.TimelinePanel)
animationMaint.TimelineVScroll:SetOrientation(Turbine.UI.Orientation.Vertical)
animationMaint.TimelineVScroll:SetWidth(10)
animationMaint.TimelineVScroll:SetTop(animationMaint.TimelineRowHeight)
animationMaint.TimelineVScroll:SetMaximum(0)
animationMaint.TimelineVScroll:SetValue(0)
animationMaint.TimelineVScroll:SetBackColor(Settings.BackColor)
animationMaint.TimelineVScroll.ValueChanged=function()
	animationMaint.TimelineHeader:SetTop(animationMaint.TimelineHeaderCaption:GetHeight()-animationMaint.TimelineVScroll:GetValue())
	animationMaint.TimelineList:SetTop(animationMaint.TimelineMeasure:GetHeight()-animationMaint.TimelineVScroll:GetValue())
end
animationMaint.TimelineHScroll=Turbine.UI.Lotro.ScrollBar()
animationMaint.TimelineHScroll:SetParent(animationMaint.TimelinePanel)
animationMaint.TimelineHScroll:SetOrientation(Turbine.UI.Orientation.Horizontal)
animationMaint.TimelineHScroll:SetHeight(10)
animationMaint.TimelineHScroll:SetMinimum(0)
animationMaint.TimelineHScroll:SetMaximum(0)
animationMaint.TimelineHScroll:SetValue(0)
animationMaint.TimelineHScroll:SetBackColor(Settings.BackColor)
animationMaint.TimelineHScroll.ValueChanged=function()
	animationMaint.TimelineMeasure:SetLeft(animationMaint.TimelineHeader:GetWidth()-animationMaint.TimelineHScroll:GetValue())
	animationMaint.TimelineList:SetLeft(animationMaint.TimelineHeader:GetWidth()-animationMaint.TimelineHScroll:GetValue())
	animationMaint.FrameScroll:ValueChanged() -- repositions marker
end
animationMaint.TimelineFrameMarker=Turbine.UI.Control()
animationMaint.TimelineFrameMarker:SetParent(animationMaint.TimelinePanel)
animationMaint.TimelineFrameMarker:SetZOrder(2) -- above TimelineList and TimelineMeasure but below TimelineHeader
animationMaint.TimelineFrameMarker:SetWidth(3)
animationMaint.TimelineFrameMarker:SetMouseVisible(False)
animationMaint.TimelineFrameMarker:SetBackColor(Turbine.UI.Color.Red)

animationMaint.CloseButton=Turbine.UI.Control()
animationMaint.CloseButton:SetParent(animationMaint.Border)
animationMaint.CloseButton:SetSize(16,16)
animationMaint.CloseButton:SetPosition(displayWidth-17,1)
animationMaint.CloseButton:SetZOrder(1)

animationMaint.Message=Turbine.UI.Label()
animationMaint.Message:SetParent(animationMaint.Border)
animationMaint.Message:SetMultiline(false)
animationMaint.Message:SetLeft(1)

if localPlayer:GetAlignment()==2 then
	animationMaint.CloseButton:SetBackground(0x41005f41)

	animationMaint.CloseButton.UpIcon=0x41005f41
	animationMaint.CloseButton.MouseEnter=function()
		animationMaint.CloseButton:SetBackground(0x41005f50)
		animationMaint.CloseButton.UpIcon=0x41005f50
	end
	animationMaint.CloseButton.MouseDown=function()
		animationMaint.CloseButton:SetBackground(0x41005f49)
	end
	animationMaint.CloseButton.MouseLeave=function()
		animationMaint.CloseButton:SetBackground(0x41005f41)
		animationMaint.CloseButton.UpIcon=0x41005f41
	end
else
	animationMaint.CloseButton:SetBackground(0x41000196)
	animationMaint.CloseButton.UpIcon=0x41000196
	animationMaint.CloseButton.MouseEnter=function()
		animationMaint.CloseButton:SetBackground(0x41000198)
		animationMaint.CloseButton.UpIcon=0x41000198
	end
	animationMaint.CloseButton.MouseDown=function()
		animationMaint.CloseButton:SetBackground(0x41000197)
	end
	animationMaint.CloseButton.MouseLeave=function()
		animationMaint.CloseButton:SetBackground(0x41000196)
		animationMaint.CloseButton.UpIcon=0x41000196
	end
end
animationMaint.CloseButton.MouseUp=function()
	animationMaint.CloseButton:SetBackground(animationMaint.CloseButton.UpIcon)
end
animationMaint.CloseButton.MouseClick=function()
	chapterMaint.MouseMask:SetMouseVisible(false)
	-- clear the spriteControls
	clearSpriteControls()
	animationMaint:SetVisible(false)
end

newSprite=function(id)
	sprite={}
	sprite.Name=tostring(id)
	sprite.Width="32"
	sprite.Height="32"
	sprite.Top="0"
	sprite.Left="0"
	sprite.Image=""
	sprite.FrameSets={}
	sprite.FrameSets[1]={}
	-- create blank initial frameSet
	sprite.FrameSets[1].StartFrame=1
	return sprite
end
addSprite=function()
	if animationMaint.TargetContent.Sprites==nil then
		animationMaint.TargetContent.Sprites={}
	end
	local spriteIndex=#animationMaint.TargetContent.Sprites+1
	table.insert(animationMaint.TargetContent.Sprites,newSprite(spriteIndex))
	newSpriteControl(spriteIndex)
	refreshFrameSetList(spriteIndex,1)
	refreshTimeline()
	authorWindow.SetChangedState(true)
	return spriteIndex
end

--releaseChildren=function(control)
--	-- this function recursively calls itself to release all child controls of a control
--	local children=control:GetControls()
--	if children~=nil and children:GetCount()>0 then
--		for k=1,children:GetCount() do
--			local child=children:Get(k)
--			child:SetVisible(false) -- hide the child until garbage collection gets around to wiping it out completely
--			releaseChildren(child)
--		end
--		children:Clear() -- removes all children (sets their parent to nil so they can be garbage collected)
--	end
--end
timelineSplitFrameSet=function(spriteIndex,index,frame)
	-- this will split the frameset[index] in Sprites[spriteIndex], creating a new frameset with a StartFrame=frame
	local sprite=animationMaint.TargetContent.Sprites[spriteIndex]
	local frameSet=sprite.FrameSets[index]
	local spriteControl=spriteControls[spriteIndex]

	-- create a new frameset
	local newFrameSet={}
	if frame==frameSet.StartFrame then frame=frame+1 end -- split at first frame becomes split at second frame
	newFrameSet.StartFrame=frame
	newFrameSet.NextFrame=frameSet.NextFrame
	frameSet.NextFrame=frame
	newFrameSet.ImageString=frameSet.ImageString
	newFrameSet.WidthString=frameSet.WidthString
	newFrameSet.HeightString=frameSet.HeightString
	newFrameSet.LeftString=frameSet.LeftString
	newFrameSet.TopString=frameSet.TopString
	newFrameSet.ZOrderString=frameSet.ZOrderString

	table.insert(sprite.FrameSets,index+1,newFrameSet)
	newFrameSetControl(spriteControl,frame,newFrameSet.NextFrame)
	authorWindow.SetChangedState(true)
end
frameSetDrag=function(currentFrameCtrl,nextFrameCtrl,sender,X)
	local delta=X-sender.X
	local spriteIndex=currentFrameCtrl:GetParent().SpriteIndex
	delta=delta-(delta % 3) -- only allow moves in increments of 3 (frames are 3 pixels wide)
	if currentFrameCtrl:GetWidth()+delta<3 then
		delta=0-currentFrameCtrl:GetWidth()+3
	end
	if nextFrameCtrl:GetWidth()-delta<3 then
		delta=nextFrameCtrl:GetWidth()-3
	end
	currentFrameCtrl:SetWidth(currentFrameCtrl:GetWidth()+delta)
	currentFrameCtrl.EndMarker:SetLeft(currentFrameCtrl:GetWidth()-2)
	nextFrameCtrl:SetWidth(nextFrameCtrl:GetWidth()-delta)
	nextFrameCtrl.EndMarker:SetLeft(nextFrameCtrl:GetWidth()-2)
	nextFrameCtrl:SetLeft(nextFrameCtrl:GetLeft()+delta)
	-- need to update underlying framesets
	local nextFrame=nextFrameCtrl:GetLeft()/3+1
	currentFrameCtrl.NextFrame=nextFrame
	nextFrameCtrl.StartFrame=nextFrame
	animationMaint.TargetContent.Sprites[spriteIndex].FrameSets[currentFrameCtrl.Index].NextFrame=nextFrame
	animationMaint.TargetContent.Sprites[spriteIndex].FrameSets[nextFrameCtrl.Index].StartFrame=nextFrame
	refreshFrameSetList(spriteIndex,animationMaint.CurrentFrameSetIndex)
	refreshAnimationPreview() -- we may have dragged across the frame scroll
	refreshCurrFrameList()
	authorWindow.SetChangedState(true)
end
timelineDeleteSprite=function(spriteIndex)
	removeSpriteControl(spriteIndex)
	table.remove(animationMaint.TargetContent.Sprites,spriteIndex)
	animationMaint.SelectedFrameSet=nil
	authorWindow.SetChangedState(true)
end
timelineDeleteFrameSet=function(spriteIndex,index)
	-- this will remove frameset[index] from Sprites[spriteIndex]
	if index==1 and animationMaint.TargetContent.Sprites[spriteIndex].FrameSets[1].NextFrame==nil then
		-- last frameset, delete entire sprite
		timelineDeleteSprite(spriteIndex)
		refreshFrameSetList()
	else
		removeFrameSetControl(spriteIndex,index)
		-- need to adjust start/next frame
		if index==1 then
			local next=animationMaint.TargetContent.Sprites[spriteIndex].FrameSets[2]
			if next~=nil then
				next.StartFrame=1
			end
		else
			local prior=animationMaint.TargetContent.Sprites[spriteIndex].FrameSets[index-1]
			prior.NextFrame=animationMaint.TargetContent.Sprites[spriteIndex].FrameSets[index].NextFrame
		end
		table.remove(animationMaint.TargetContent.Sprites[spriteIndex].FrameSets,index)
		refreshFrameSetList(spriteIndex,index-1)
	end
	refreshCurrFrameList()
	authorWindow.SetChangedState(true)
end
timeLineMenu=Turbine.UI.ContextMenu()
timeLineMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][283]));
timeLineMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][288]));

timeLineMenu:GetItems():Get(1).Click=function(sender,args)
	animationMaint.CurrentSpriteIndex=addSprite()
	spriteControls[animationMaint.CurrentSpriteIndex].List.FrameSets[1]:MouseClick({["Button"]=1})
end
timeLineMenu:GetItems():Get(2).Click=function(sender,args)
	animationMaint.CurrentSpriteIndex=addSprite()
	spriteControls[animationMaint.CurrentSpriteIndex].List.FrameSets[1]:MouseClick({["Button"]=1})
	-- now select the added sprite and paste
	pasteSpriteFromClipboard(animationMaint.CurrentSpriteIndex)
end
animationMaint.TimelinePanel.MouseClick=function(sender,args)
	if args.Button==2 then
		timeLineMenu:GetItems():Get(2):SetEnabled(clipboard.Type=="Sprite")
		timeLineMenu:ShowMenu()
	end
end
refreshAnimationPreview=function()
	local background=animationMaint.BackImage:GetText()
	local width=tonumber(animationMaint.Width:GetText())
	local height=tonumber(animationMaint.Height:GetText())
	local maxWidth=animationMaint.PreviewPanel:GetWidth()
	local maxHeight=animationMaint.PreviewPanel:GetHeight()
	local frame=animationMaint.FrameScroll:GetValue()
	local prevWidth=maxWidth
	local prevHeight=maxHeight
	if background~="" then
		animationMaint.Width:SetEnabled(false)
		animationMaint.Height:SetEnabled(false)
		animationMaint.PreviewBackground:SetStretchMode(0)
		-- determine actual size and aspect ratio
		if background=="" then background=nil end
		if tonumber(background)~=nil and tonumber(background)>0 then
			background=tonumber(background)
		elseif background~=nil then
			local path=animationMaint.ImagePath:GetText()
			if path~="" then
				if string.sub(path,-1)~="/" then path=path.."/" end
			end
			background=path..background
		end
		local success,result=pcall(Turbine.UI.Control.SetBackground,animationMaint.PreviewBackground,background)
		if success then
			-- if a background image is used, it sets the size of the animation
			animationMaint.PreviewBackground:SetStretchMode(2)
			width,height=animationMaint.PreviewBackground:GetSize()
			animationMaint.Width:SetText(tostring(width))
			animationMaint.Height:SetText(tostring(height))
			animationMaint.TargetContent.Width=width
			animationMaint.TargetContent.Height=height
		else
			animationMaint.Message:SetText(tostring(result))
		end
		animationMaint.PreviewBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Color)
		if animationMaint.UseBackColor:IsChecked() then
			animationMaint.PreviewBackground:SetBackColor(animationMaint.BackColor:GetColor())
		else
			animationMaint.PreviewBackground:SetBackColor(nil)
		end
	else
		animationMaint.Width:SetEnabled(true)
		animationMaint.Height:SetEnabled(true)
		animationMaint.PreviewBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Normal)
		if animationMaint.UseBackColor:IsChecked() then
			animationMaint.PreviewBackground:SetBackColor(animationMaint.BackColor:GetColor())
		else
			animationMaint.PreviewBackground:SetBackColor(Settings.BackColor)
		end
	end

	if width==nil then width=maxWidth end
	if height==nil then height=maxHeight end
	local aspect=width/height
	local previewAspect=maxWidth/maxHeight
	if aspect<previewAspect then
		maxWidth=aspect*maxHeight
	else
		maxHeight=maxWidth/aspect
	end

	animationMaint.PreviewBackground:SetStretchMode(0)
	animationMaint.PreviewBackground:SetSize(width,height)
	-- refresh sprites
	if animationMaint.TargetContent~=nil then
		for k,v in ipairs(animationMaint.TargetContent.Sprites) do
			if spriteControls[k]~=nil and spriteControls[k].CurrentImage~=nil then
				-- determine frameset
				local frameset
				for index,tmpFrameSet in ipairs(v.FrameSets) do
					if tmpFrameSet.StartFrame<=frame and (tmpFrameSet.NextFrame==nil or tmpFrameSet.NextFrame>frame) then
						frameset=tmpFrameSet
						break
					end
				end
				local visible=false
				if frameset~=nil and spriteControls[k].Header.CheckBox:IsChecked() then
					local imageOK=false
					local background
					local spriteWidth,spriteHeight
					local startFrame,endFrame=frameset.StartFrame,frameset.NextFrame
					if endFrame==nil then endFrame=animationMaint.TargetContent.FrameCount else endFrame=endFrame-1 end
					if v.FixedSize then
						spriteWidth=tonumber(v.Width)
						spriteHeight=tonumber(v.Height)
					end
					if spriteWidth==nil and frameset.WidthString~=nil and frameset.WidthString~="" then
						spriteWidth=tonumber(frameEval(frameset.WidthString,frame,startFrame,endFrame))
					end
					if spriteHeight==nil and frameset.HeightString~=nil and frameset.HeightString~="" then
						spriteHeight=tonumber(frameEval(frameset.HeightString,frame,startFrame,endFrame))
					end
					if frameset.ImageString~=nil and frameset.ImageString~="" then
						-- evaluate image string for this frame
						background=frameEval(frameset.ImageString,frame,startFrame,endFrame)
						if background~=nil then
							if type(background)=="string" then
								local path=animationMaint.TargetContent.Path
								if path~="" and string.sub(path,-1)~="/" then path=path.."/" end
								background=path..background
							end
							local result
							imageOK,result=pcall(Turbine.UI.Control.SetBackground,spriteControls[k].CurrentImage,background)
						end
					end
					local left,top,zorder
					if frameset.LeftString~=nil and frameset.LeftString~="" then
						left=tonumber(frameEval(frameset.LeftString,frame,startFrame,endFrame))
					end
					if left==nil then left=0 end
					if frameset.TopString~=nil and frameset.TopString~="" then
						top=tonumber(frameEval(frameset.TopString,frame,startFrame,endFrame))
					end
					if top==nil then top=0 end
					spriteControls[k].CurrentImage:SetPosition(left,top)
					if frameset.ZOrderString~=nil and frameset.ZOrderString~="" then
						zorder=tonumber(frameEval(frameset.ZOrderString,frame,startFrame,endFrame))
					end
					if zorder==nil then zorder=0 end
					spriteControls[k].CurrentImage:SetZOrder(zorder)
					local dynSize=false
					if imageOK then
						if spriteWidth==nil or spriteHeight==nil then
							dynSize=true
							if background~=nil then
								-- get from image
								local tmpImage=Turbine.UI.Control()
								tmpImage:SetBackground(background)
								tmpImage:SetStretchMode(2)
								if spriteWidth==nil then spriteWidth=tmpImage:GetWidth() end
								if spriteHeight==nil then spriteHeight=tmpImage:GetHeight() end
								tmpImage=nil
							end
						end
						spriteControls[k].CurrentImage:SetBackColorBlendMode(Turbine.UI.BlendMode.Color)
						visible=true
					else
						spriteControls[k].CurrentImage:SetBackColorBlendMode(Turbine.UI.BlendMode.Normal)
						spriteControls[k].CurrentImage:SetBackground(nil)
						-- don't change visible yet since without and image or text the control should be hidden
					end
					local backColor=frameset.BackColor
					if backColor==nil then
						spriteControls[k].CurrentImage:SetBackColor(nil)
					else
						spriteControls[k].CurrentImage:SetBackColor(Turbine.UI.Color(backColor.R,backColor.G,backColor.B))
					end
					if frameset.TextString~=nil and frameset.TextString~="" then
						-- control has text
						local text=frameEval(frameset.TextString,frame,startFrame,endFrame)
						local foreColor=frameset.ForeColor
						if foreColor==nil then
							spriteControls[k].CurrentImage:SetForeColor(Settings.FontColor)
						else
							spriteControls[k].CurrentImage:SetForeColor(Turbine.UI.Color(foreColor.R,foreColor.G,foreColor.B))
						end
						local font=frameset.Font
						if font==nil then
							spriteControls[k].CurrentImage:SetFont(Settings.Font)
							fontMetric:SetFont(Settings.Font)
						else
							spriteControls[k].CurrentImage:SetFont(font)
							fontMetric:SetFont(font)
						end
						if spriteWidth==nil or dynSize then
							local tmpWidth=fontMetric:GetTextWidth(text)
							if tmpWidth>displayWidth then tmpWidth=displayWidth end
							if spriteWidth==nil or tmpWidth>spriteWidth then spriteWidth=tmpWidth end
						end
						if spriteHeight==nil or dynSize then
							local tmpHeight=fontMetric:GetTextHeight(text,spriteWidth)
							if tmpHeight>displayHeight then tmpHeight=displayHeight end
							if spriteHeight==nil or tmpHeight>spriteHeight then spriteHeight=tmpHeight end
						end
						spriteControls[k].CurrentImage:SetText(text)
						visible=true
					else
						spriteControls[k].CurrentImage:SetText("")
					end
					if spriteWidth~=nil and spriteHeight~=nil then
						spriteControls[k].CurrentImage:SetSize(spriteWidth,spriteHeight)
					else
						visible=false
					end
				end
				spriteControls[k].CurrentImage:SetVisible(visible)
			end
		end
	end

	-- now set stretchmode to control and all of it's children
	if width>maxWidth or height>maxHeight then
		animationMaint.PreviewBackground:SetBlendMode(Turbine.UI.BlendMode.Normal)
		animationMaint.PreviewBackground:SetStretchMode(1)
		animationMaint.PreviewBackground:SetSize(maxWidth,maxHeight)
		width=maxWidth
		height=maxHeight
	else
		animationMaint.PreviewBackground:SetBlendMode(Turbine.UI.BlendMode.Overlay)
	end
	if width<prevWidth then
		animationMaint.PreviewBackground:SetLeft((prevWidth-width)/2)
	else
		animationMaint.PreviewBackground:SetLeft(0)
	end
	if height<prevHeight then
		animationMaint.PreviewBackground:SetTop((prevHeight-height)/2)
	else
		animationMaint.PreviewBackground:SetTop(0)
	end
end
refreshTimeline=function()
	-- reposition the timeline elements based on scroll values and current frame
	local hScroll=animationMaint.TimelineHScroll:GetValue()
	local vScroll=animationMaint.TimelineVScroll:GetValue()
	local spriteIndex=animationMaint.CurrentSpriteIndex
	local frameSet=animationMaint.CurrentFrameSetIndex
	local frame=animationMaint.FrameScroll:GetValue()
	local frameCount=tonumber(animationMaint.FrameCount:GetText())
	local spriteCount=0
	if chapterMaint.ChapterID~=nil and authorWindow.CurrLanguage:GetValue()~=nil and authorStory.Chapters[chapterMaint.ChapterID].Content[animationMaint.TargetControl.Index][authorWindow.CurrLanguage:GetValue()].Sprites~=nil then
		spriteCount=#authorStory.Chapters[chapterMaint.ChapterID].Content[animationMaint.TargetControl.Index][authorWindow.CurrLanguage:GetValue()].Sprites
	end
	if frameCount==nil then frameCount=0 end
	animationMaint.TimelineList:SetWidth(frameCount*3)
	animationMaint.TimelineMeasure:SetWidth(frameCount*3)
	animationMaint.TimelineHeader:SetHeight((spriteCount)*(animationMaint.TimelineRowHeight+1))
	animationMaint.TimelineList:SetHeight(animationMaint.TimelineHeader:GetHeight())

--*** what if there is an existing frameset with StartFrame that is past the new FrameCount???
	if animationMaint.TargetContent~=nil and animationMaint.TargetContent.Sprites~=nil then
		for k,v in ipairs(spriteControls) do
			local row=v.List
			row:SetWidth(animationMaint.TimelineList:GetWidth())
			for index,fs in ipairs(row.FrameSets) do
				if fs.NextFrame==nil then
					fs:SetWidth(animationMaint.TimelineList:GetWidth()-fs:GetLeft())
					fs.EndMarker:SetLeft(fs:GetWidth()-2)
				end
			end
		end
	end

	local max=animationMaint.TimelineHeader:GetHeight()
	if animationMaint.TimelineVScroll:GetValue()>max then animationMaint.TimelineVScroll:SetValue(max) end
	animationMaint.TimelineVScroll:SetMaximum(max)
	local max=animationMaint.TimelineList:GetWidth()
	if animationMaint.TimelineHScroll:GetValue()>max then animationMaint.TimelineHScroll:SetValue(max) end
	animationMaint.TimelineHScroll:SetMaximum(max)
	local left=animationMaint.TimelineHeader:GetWidth()+frame*3-animationMaint.TimelineHScroll:GetValue()-3
	animationMaint.TimelineFrameMarker:SetVisible(left>=animationMaint.TimelineHeader:GetWidth())
	animationMaint.TimelineFrameMarker:SetLeft(left)
	animationMaint.TimelineMeasure:SetLeft(animationMaint.TimelineHeader:GetWidth()-animationMaint.TimelineHScroll:GetValue())
	animationMaint.TimelineList:SetLeft(animationMaint.TimelineHeader:GetWidth()-animationMaint.TimelineHScroll:GetValue())
	animationMaint.TimelineHeader:SetTop(animationMaint.TimelineHeaderCaption:GetHeight()-animationMaint.TimelineVScroll:GetValue())
	animationMaint.TimelineList:SetTop(animationMaint.TimelineMeasure:GetHeight()-animationMaint.TimelineVScroll:GetValue())
end
initFrameSetList=function()
	refreshingFrameSetList=true
	local controlHeight=Turbine.UI.Lotro.FontInfo[Settings.Font].size+4
	fontMetric:SetFont(Settings.Font)

	local row={} -- {resourceID, control type, field name, field level(Sprite=1,FrameSet=2)}
	row[1]={269,"label","Name",1,"string"}
	row[2]={276,"check","FixedSize",1,"boolean"}
	row[3]={263,"text","Width",1,"number"}
	row[4]={264,"text","Height",1,"number"}
	row[5]={nil,"sep","Seperator",nil,nil}
	row[6]={270,"label","Index",nil,"string"}
	row[7]={275,"label","StartFrame",2,"number"}
	row[8]={271,"text","ImageString",2,"string"}
	row[9]={263,"text","WidthString",2,"string"}
	row[10]={264,"text","HeightString",2,"string"}
	row[11]={272,"text","LeftString",2,"string"}
	row[12]={273,"text","TopString",2,"string"}
	row[13]={274,"text","ZOrderString",2,"string"}
	row[14]={92,"text","TextString",2,"string"}
	row[15]={49,"font","Font",2,"number"}
	row[16]={267,"color","BackColor",2,"color"}
	row[17]={290,"color","ForeColor",2,"color"}

	animationMaint.spriteWidthRow=3
	animationMaint.spriteHeightRow=4
	animationMaint.frameSetWidthStringRow=9
	animationMaint.frameSetHeightStringRow=10
	animationMaint.frameSetTextStringRow=14
	animationMaint.frameSetFontRow=15
	animationMaint.frameSetBackColorRow=16
	animationMaint.frameSetForeColorRow=17

	local lblWidth=0
	for k,v in ipairs(row) do
		if v[1]~=nil then
			local tmp=fontMetric:GetTextWidth(Resource[language][v[1]]..":")
			if tmp>lblWidth then lblWidth=tmp end
		end
	end
	animationMaint.FrameSetList:ClearItems()
	for k,v in ipairs(row) do
		local tmpRow=Turbine.UI.Control()
		tmpRow:SetParent(animationMaint.FrameSetList)
		tmpRow:SetSize(lblWidth+300,controlHeight)
		tmpRow.Type=v[2]
		tmpRow.Name=v[3]
		tmpRow.Level=v[4]
		tmpRow.DataType=v[5]
		if v[2]=="check" then
			tmpRow.Check=Turbine.UI.Lotro.CheckBox()
			tmpRow.Check:SetParent(tmpRow)
			tmpRow.Check:SetSize(tmpRow:GetWidth()-20,controlHeight)
			tmpRow.Check:SetLeft(20)
			tmpRow.Check:SetForeColor(Settings.FontColor)
			tmpRow.Check:SetText(Resource[language][276])
			tmpRow.Check.CheckedChanged=function(sender,args)
				local row=sender:GetParent()
				local isChecked=sender:IsChecked()
				if row.Name=="FixedSize" then -- special handling for "FixedSize" setting
					animationMaint.FrameSetList:GetItem(animationMaint.spriteWidthRow).Value:SetVisible(isChecked)
					animationMaint.FrameSetList:GetItem(animationMaint.spriteHeightRow).Value:SetVisible(isChecked)
					animationMaint.FrameSetList:GetItem(animationMaint.frameSetWidthStringRow).Value:SetVisible(not isChecked)
					animationMaint.FrameSetList:GetItem(animationMaint.frameSetHeightStringRow).Value:SetVisible(not isChecked)
				end
				local row=sender:GetParent()
				if animationMaint.TargetContent~=nil and animationMaint.CurrentSpriteIndex~=nil and animationMaint.CurrentFrameSetIndex~=nil then
					local sprite=animationMaint.TargetContent.Sprites[animationMaint.CurrentSpriteIndex]
					local frameSet=sprite.FrameSets[animationMaint.CurrentFrameSetIndex]
					if row.Level==1 then
						sprite[row.Name]=isChecked
					else
						frameSet[row.Name]=isChecked
					end
				end
				if not refreshingFrameSetList then
					authorWindow.SetChangedState(true)
				end
			end
		elseif v[2]=="sep" then
			tmpRow.Sep=Turbine.UI.Control()
			tmpRow.Sep:SetParent(tmpRow)
			tmpRow.Sep:SetBackColor(Settings.TrimColor)
			tmpRow.Sep:SetSize(tmpRow:GetWidth()-10,1)
			tmpRow.Sep:SetPosition(5,controlHeight/2)
		elseif v[2]=="font" then
			tmpRow.Label=Turbine.UI.Label()
			tmpRow.Label:SetParent(tmpRow)
			tmpRow.Label:SetSize(lblWidth,controlHeight)
			tmpRow.Label:SetFont(Settings.Font)
			tmpRow.Label:SetForeColor(Settings.LabelColor)
			tmpRow.Label:SetText(Resource[language][v[1]]..":")
			tmpRow.FontText=Turbine.UI.Label()
			tmpRow.FontText:SetParent(tmpRow)
			tmpRow.FontText:SetSize(tmpRow:GetWidth()-lblWidth,tmpRow:GetHeight())
			tmpRow.FontText:SetFont(Settings.Font)
			tmpRow.FontText:SetLeft(lblWidth)
			tmpRow.FontText.Font=Settings.Font
			tmpRow.FontText:SetText(Turbine.UI.Lotro.FontInfo[tmpRow.FontText.Font].name.." "..tostring(Turbine.UI.Lotro.FontInfo[tmpRow.FontText.Font].size))
			tmpRow.FontText.MouseClick=function(sender,args)
				if sender:IsEnabled() then
					local row=sender:GetParent()
					if sender.Font~=nil then
						row.Font:SetFont(sender.Font)
					end
					local args2={}
					args2.Button=args.Button
					args2.X=args.X
					args2.Y=sender:GetParent():GetTop()+args.Y
					row.Font:MouseClick(args2)
				end
			end
			tmpRow.FontText:SetEnabled(false)
			-- need fontselect
			tmpRow.Font=FontSelect()
			tmpRow.Font.Row=tmpRow
			tmpRow.Font:SetParent(animationMaint.FrameSetList)
			tmpRow.Font:SetFont(Settings.Font) -- default to current font
			tmpRow.Font.FontChanged=function(sender,args)
				if not refreshingFrameSetList and sender.Row.FontText:IsEnabled() then
					local sprite=animationMaint.TargetContent.Sprites[animationMaint.CurrentSpriteIndex]
					local frameSet=sprite.FrameSets[animationMaint.CurrentFrameSetIndex]
					sender.Row.FontText.Font=sender:GetFont()
					if sender.Row.Level==1 then
						sprite[sender.Row.Name]=sender.Row.FontText.Font
					else
						frameSet[sender.Row.Name]=sender.Row.FontText.Font
					end
					sender.Row.FontText:SetText(Turbine.UI.Lotro.FontInfo[sender.Row.FontText.Font].name.." "..tostring(Turbine.UI.Lotro.FontInfo[sender.Row.FontText.Font].size))
					refreshAnimationPreview()
					refreshCurrFrameList()
					authorWindow.SetChangedState(true)
				end
			end
		elseif v[2]=="color" then
			tmpRow.Label=Turbine.UI.Label()
			tmpRow.Label:SetParent(tmpRow)
			tmpRow.Label:SetSize(lblWidth,controlHeight)
			tmpRow.Label:SetFont(Settings.Font)
			tmpRow.Label:SetForeColor(Settings.LabelColor)
			tmpRow.Label:SetText(Resource[language][v[1]]..":")
			tmpRow.Check=Turbine.UI.Lotro.CheckBox()
			tmpRow.Check:SetParent(tmpRow)
			tmpRow.Check:SetSize(20,20)
			tmpRow.Check:SetPosition(lblWidth+5,tmpRow:GetHeight()/2-10)
			tmpRow.Check:SetForeColor(Settings.FontColor)
			tmpRow.Check:SetText(" ")
			tmpRow.Check:SetChecked(false)
			tmpRow.Check:SetEnabled(false)
			tmpRow.Check.CheckedChanged=function(sender,args)
				sender:GetParent().ColorPicker:SetVisible(sender:IsChecked())
				if not refreshingFrameSetList then
					local sprite=animationMaint.TargetContent.Sprites[animationMaint.CurrentSpriteIndex]
					local frameSet=sprite.FrameSets[animationMaint.CurrentFrameSetIndex]
					local row=sender:GetParent()
					if sender:IsChecked() then
						if row.Level==1 then
							sprite[row.Name]=row.ColorPicker:GetColor()
						else
							frameSet[row.Name]=row.ColorPicker:GetColor()
						end
					else
						if row.Level==1 then
							sprite[row.Name]=nil
						else
							frameSet[row.Name]=nil
						end
					end
					refreshAnimationPreview()
					refreshCurrFrameList()
					authorWindow.SetChangedState(true)
				end
			end
			-- create color picker
			tmpRow.ColorPicker=ColorPicker()
			tmpRow.ColorPicker:SetParent(tmpRow)
			tmpRow.ColorPicker:SetLeft(lblWidth+30)
			tmpRow.ColorPicker:SetSize(tmpRow:GetWidth()-lblWidth-30,tmpRow:GetHeight())
			tmpRow.ColorPicker:SetEnabled(false)
			tmpRow.ColorPicker:SetVisible(false)
			if tmpRow.Name=="BackColor" then
				tmpRow.ColorPicker:SetColor(Settings.BackColor)
			else
				tmpRow.ColorPicker:SetColor(Settings.FontColor)
			end
			tmpRow.ColorPicker.ColorChanged=function(sender,args)
				if not refreshingFrameSetList and sender:IsEnabled() then
					local sprite=animationMaint.TargetContent.Sprites[animationMaint.CurrentSpriteIndex]
					local frameSet=sprite.FrameSets[animationMaint.CurrentFrameSetIndex]
					local row=sender:GetParent()
					if row.Check:IsChecked() then
						if row.Level==1 then
							sprite[row.Name]=sender:GetColor()
						else
							frameSet[row.Name]=sender:GetColor()
						end
					else
						if row.Level==1 then
							sprite[row.Name]=nil
						else
							frameSet[row.Name]=nil
						end
					end
					refreshAnimationPreview()
					refreshCurrFrameList()
					authorWindow.SetChangedState(true)
				end
			end
		else
			tmpRow.Label=Turbine.UI.Label()
			tmpRow.Label:SetParent(tmpRow)
			tmpRow.Label:SetSize(lblWidth,controlHeight)
			tmpRow.Label:SetFont(Settings.Font)
			tmpRow.Label:SetForeColor(Settings.LabelColor)
			tmpRow.Label:SetText(Resource[language][v[1]]..":")
			if v[2]=="label" then
				tmpRow.Value=Turbine.UI.Label()
			else
				tmpRow.Value=Turbine.UI.Lotro.TextBox()
				tmpRow.Value.KeyDown=function(sender,args)
					if args.Action==75 then
						tmpRow.Value:TextChanged()
					end
					if args.Action==162 then -- enter key
						tmpRow.Value.FocusLost()
					end
				end
			end
			tmpRow.Value:SetMultiline(false)
			tmpRow.Value:SetParent(tmpRow)
			tmpRow.Value:SetLeft(lblWidth)
			tmpRow.Value:SetSize(300,controlHeight)
			tmpRow.Value:SetFont(Settings.Font)
			tmpRow.Value:SetMultiline(false)
			tmpRow.Value:SetForeColor(Settings.FontColor)
			tmpRow.Value.FocusLost=function()
				refreshAnimationPreview()
				refreshCurrFrameList()
			end
			tmpRow.Value.TextChanged=function(sender,args)
				if not refreshingFrameSetList then
					local row=sender:GetParent()
					if animationMaint.TargetContent~=nil and animationMaint.CurrentSpriteIndex~=nil and animationMaint.CurrentFrameSetIndex~=nil then
						local sprite=animationMaint.TargetContent.Sprites[animationMaint.CurrentSpriteIndex]
						local frameSet=sprite.FrameSets[animationMaint.CurrentFrameSetIndex]
						if row.Level==1 then
							if tmpRow.DataType=="string" then
								sprite[row.Name]=sender:GetText()
							elseif tmpRow.DataType=="number" then
								sprite[row.Name]=tonumber(sender:GetText())
							elseif tmpRow.DataType=="boolean" then
								sprite[row.Name]=toboolean(sender:GetText()) -- "nil" will return nil, any value other than "false" will be considered true since GetText is a string, even "" is considered true
							end
						else
							if tmpRow.DataType=="string" then
								frameSet[row.Name]=sender:GetText()
							elseif tmpRow.DataType=="number" then
								frameSet[row.Name]=tonumber(sender:GetText())
							elseif tmpRow.DataType=="boolean" then
								frameSet[row.Name]=toboolean(sender:GetText()) -- "nil" will return nil, any value other than "false" will be considered true since GetText is a string, even "" is considered true
							end
						end
					end
					authorWindow.SetChangedState(true)
				end
			end
		end
		animationMaint.FrameSetList:AddItem(tmpRow)
	end
	refreshingFrameSetList=false
end
initFrameSetList()
animationMaint.FrameSetList.SizeChanged=function()
	local width=animationMaint.FrameSetList:GetWidth()
	for k=1,animationMaint.FrameSetList:GetItemCount() do
		local tmpRow=animationMaint.FrameSetList:GetItem(k)
		tmpRow:SetWidth(width)
		if tmpRow.Sep~=nil then
			tmpRow.Sep:SetWidth(width-10)
		end
		if tmpRow.Type=="color" then
			local newWidth=width-tmpRow.ColorPicker:GetLeft()
			if newWidth<0 then newWidth=0 end
			tmpRow.ColorPicker:SetWidth(newWidth)
		elseif tmpRow.Type=="font" then
			local newWidth=width-tmpRow.FontText:GetLeft()
			if newWidth<0 then newWidth=0 end
			tmpRow.FontText:SetWidth(newWidth)
		else
			if tmpRow.Value~=nil then
				local newWidth=width-tmpRow.Value:GetLeft()
				if newWidth<0 then newWidth=0 end
				tmpRow.Value:SetWidth(newWidth)
			end
		end
	end
end

refreshFrameSetList=function(spriteIndex,setIndex)
	refreshingFrameSetList=true
	if spriteIndex==nil then
		spriteIndex=animationMaint.CurrentSpriteIndex
	else
		animationMaint.CurrentSpriteIndex=spriteIndex
	end
	if spriteIndex==nil then
		for k=1,animationMaint.FrameSetList:GetItemCount() do
			local item=animationMaint.FrameSetList:GetItem(k)
			if item.Type=="check" then
				item.Check:SetChecked(false)
				item.Check:CheckedChanged() -- fire event incase the value happens to be the same
				item.Check:SetEnabled(false)
			elseif item.Type=="label" or item.Type=="text" then
				item.Value:SetText("")
				item.Value:SetEnabled(false)
			end
		end
	else
		if setIndex==nil then
			-- determine from frameScroll
			setIndex=1
			local frame=animationMaint.FrameScroll:GetValue()
			for k,v in ipairs(animationMaint.TargetContent.Sprites[spriteIndex].FrameSets) do
				if v.NextFrame~=nil and frame>=v.NextFrame then break end
				if v.StartFrame<=frame and (v.NextFrame==nil or v.NextFrame>frame) then
					setIndex=k
					break
				end
			end
		end
		animationMaint.CurrentFrameSetIndex=setIndex
		local sprite=animationMaint.TargetContent.Sprites[spriteIndex]
		local frameSet
		if sprite~=nil then
			frameSet=sprite.FrameSets[setIndex]
		end
		-- fill in listbox values
		for k=1,animationMaint.FrameSetList:GetItemCount() do
			local item=animationMaint.FrameSetList:GetItem(k)
			if item.Type=="check" then
				item.Check:SetEnabled(true)
				if item.Level==1 then
					if sprite~=nil then
						item.Check:SetChecked(sprite[item.Name])
					else
						item.Check:SetChecked(false)
					end
				else
					if frameSet~=nil then
						item.Check:SetChecked(frameSet[item.Name])
					else
						item.Check:SetChecked(false)
					end
				end
				item.Check:CheckedChanged() -- fire event incase the value happens to be the same
			elseif item.Type=="font" then
				item.FontText:SetEnabled(true)
				if item.Level==1 then
					if sprite~=nil then
						item.FontText.Font=tonumber(sprite[item.Name])
					end
				else
					if frameSet~=nil then
						item.FontText.Font=tonumber(frameSet[item.Name])
					end
				end
				if item.FontText.Font==nil then item.FontText.Font=Settings.Font end
				item.FontText:SetText(Turbine.UI.Lotro.FontInfo[item.FontText.Font].name.." "..tostring(Turbine.UI.Lotro.FontInfo[item.FontText.Font].size))
			elseif item.Type=="color" then
				item.Check:SetEnabled(true)
				item.ColorPicker:SetEnabled(true)
				local color
				if item.Level==1 then
					if sprite~=nil then
						color=sprite[item.Name]
					end
				else
					if frameSet~=nil then
						color=frameSet[item.Name]
					end
				end
				if color==nil then
					item.Check:SetChecked(false)
				else
					item.Check:SetChecked(true)
					item.ColorPicker:SetColor(Turbine.UI.Color(color.R,color.G,color.B)) --coerce into a color incase it was just read from .plugindata
				end
			elseif item.Type=="label" or item.Type=="text" then
				item.Value:SetEnabled(true)
				local value
				if item.Name=="Index" then
					value=setIndex
				elseif item.Level==1 then
					if sprite~=nil then
						value=sprite[item.Name]
					else
						value="nil"
					end
				else
					if frameSet~=nil then
						value=frameSet[item.Name]
					else
						value="nil"
					end
				end
				if value==nil then
					item.Value:SetText("")
				else
					item.Value:SetText(value)
				end
			end
		end
	end
	refreshingFrameSetList=false
end
toolTip=Turbine.UI.ContextMenu()
toolTip:GetItems():Add(Turbine.UI.MenuItem(""))
toolTip:GetItems():Get(1):SetEnabled(false)

initCurrFrameList=function()
	local controlHeight=Turbine.UI.Lotro.FontInfo[Settings.Font].size+4
	fontMetric:SetFont(Settings.Font)

	local row={} -- {resourceID, control type, field name, field level(Sprite=1,FrameSet=2)}
	row[1]={277,"header","Header"}
	row[2]={278,"label","Frame"}
	row[3]={269,"label","Sprite"}
	row[4]={271,"label","Image"}
	row[5]={263,"label","Width"}
	row[6]={264,"label","Height"}
	row[7]={272,"label","Left"}
	row[8]={273,"label","Top"}
	row[9]={274,"label","ZOrder"}
	row[10]={92,"label","TextString"}
	row[11]={49,"label","Font"}
	row[12]={267,"color","BackColor"}
	row[13]={290,"color","ForeColor"}

	local lblWidth=0
	for k,v in ipairs(row) do
		if v[1]~=nil then
			local tmp=fontMetric:GetTextWidth(Resource[language][v[1]]..":")
			if tmp>lblWidth then lblWidth=tmp end
		end
	end
	animationMaint.CurrFrameList:ClearItems()
	for k,v in ipairs(row) do
		local tmpRow=Turbine.UI.Control()
		tmpRow:SetParent(animationMaint.CurrFrameList)
		tmpRow:SetSize(animationMaint.CurrFrameList:GetWidth(),controlHeight)
		tmpRow.Type=v[2]
		tmpRow.Name=v[3]
		if v[2]=="sep" then
			tmpRow.Sep=Turbine.UI.Control()
			tmpRow.Sep:SetParent(tmpRow)
			tmpRow.Sep:SetBackColor(Settings.TrimColor)
			tmpRow.Sep:SetSize(tmpRow:GetWidth()-10,1)
			tmpRow.Sep:SetPosition(5,controlHeight/2)
		elseif v[2]=="header" then
			tmpRow.Header=Turbine.UI.Label()
			tmpRow.Header:SetParent(tmpRow)
			tmpRow.Header:SetSize(tmpRow:GetWidth(),controlHeight)
			tmpRow.Header:SetFont(Settings.Font)
			tmpRow.Header:SetForeColor(Settings.HeadingColor)
			tmpRow.Header:SetMultiline(false)
			tmpRow.Header:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
			tmpRow.Header:SetText(Resource[language][v[1]])
		elseif v[2]=="color" then
			tmpRow.Label=Turbine.UI.Label()
			tmpRow.Label:SetParent(tmpRow)
			tmpRow.Label:SetSize(lblWidth,controlHeight)
			tmpRow.Label:SetFont(Settings.Font)
			tmpRow.Label:SetForeColor(Settings.LabelColor)
			tmpRow.Label:SetText(Resource[language][v[1]]..":")
			tmpRow.Value=Turbine.UI.Label()
			tmpRow.Value:SetSelectable(true)
			tmpRow.Value:SetParent(tmpRow)
			tmpRow.Value:SetLeft(lblWidth)
			local valWidth=tmpRow:GetWidth()-lblWidth
			if valWidth<0 then valWidth=0 end
			tmpRow.Value:SetSize(valWidth,controlHeight)
			tmpRow.Value:SetMultiline(false)
			tmpRow.Value:SetFont(Settings.Font)
			tmpRow.Value:SetBackColor(Settings.BackColor)
		else
			tmpRow.Label=Turbine.UI.Label()
			tmpRow.Label:SetParent(tmpRow)
			tmpRow.Label:SetSize(lblWidth,controlHeight)
			tmpRow.Label:SetFont(Settings.Font)
			tmpRow.Label:SetForeColor(Settings.LabelColor)
			tmpRow.Label:SetText(Resource[language][v[1]]..":")
			tmpRow.Value=Turbine.UI.Label()
			tmpRow.Value:SetSelectable(true)
			tmpRow.Value:SetParent(tmpRow)
			tmpRow.Value:SetLeft(lblWidth)
			local valWidth=tmpRow:GetWidth()-lblWidth
			if valWidth<0 then valWidth=0 end
			tmpRow.Value:SetSize(valWidth,controlHeight)
			tmpRow.Value:SetMultiline(false)
			tmpRow.Value:SetFont(Settings.Font)
			tmpRow.Value:SetForeColor(Settings.FontColor)
			tmpRow.Value.MouseHover=function(sender,args)
				-- show value in a popup menu (tooltip)
				toolTip:GetItems():Get(1):SetText(sender:GetText())
				local x,y=Turbine.UI.Display:GetMousePosition()
				toolTip:ShowMenuAt(x,y-args.Y-20)
			end
		end
		animationMaint.CurrFrameList:AddItem(tmpRow)
	end
end
initCurrFrameList()
animationMaint.CurrFrameList.SizeChanged=function()
	local width=animationMaint.CurrFrameList:GetWidth()
	for k=1,animationMaint.CurrFrameList:GetItemCount() do
		local tmpRow=animationMaint.CurrFrameList:GetItem(k)
		tmpRow:SetWidth(width)
		if tmpRow.Sep~=nil then
			tmpRow.Sep:SetWidth(width-10)
		end
		if tmpRow.Header~=nil then
			tmpRow.Header:SetWidth(width)
		end
		if tmpRow.Value~=nil then
			local newWidth=width-tmpRow.Value:GetLeft()
			if newWidth<0 then newWidth=0 end
			tmpRow.Value:SetWidth(newWidth)
		end
	end
end
frameEval=function(tmpString,frame,startFrame,endFrame)
	-- function to compile a string as a lua function and return the result of executing the string
	local val
	if frame==nil then frame=animationMaint.FrameScroll:GetValue() end
	if startFrame==nil then startFrame=1 end
	if endFrame==nil then endFrame=animationMaint.TargetContent.FrameCount end
	if tmpString~=nil then
		local tmpFunc,error=loadstring("return "..tmpString)
		if error~=nil then
			val="Error:"..error
		else
			-- set environment to current plugin plus the local "frame" variable
			local env=getfenv()
			env["frame"]=frame
			env["startFrame"]=startFrame
			env["endFrame"]=endFrame
			setfenv(tmpFunc,env)
			-- we
			local success,result=pcall(tmpFunc)
			if success then
				val=result
			else
				val="Error:"..result
			end
		end
	end
	return val
end

refreshCurrFrameList=function(frame)
	if animationMaint.CurrentSpriteIndex~=nil then
		if frame==nil then frame=animationMaint.FrameScroll:GetValue() end
		-- apply the current formulas and display the values
		local sprite=animationMaint.TargetContent.Sprites[animationMaint.CurrentSpriteIndex]
		-- determine the frameSet based on frame and sprite.FrameSets
		local frameSet
		local frameSetIndex
		if sprite~=nil then
			for k,v in ipairs(sprite.FrameSets) do
				if v.StartFrame<=frame and (v.NextFrame==nil or v.NextFrame>frame) then
					frameSet=v
					frameSetIndex=k
					break
				end
			end
		end
		if frameSet~=nil then
			local startFrame,endFrame=frameSet.StartFrame,frameSet.NextFrame
			if endFrame==nil then endFrame=animationMaint.TargetContent.FrameCount else endFrame=endFrame-1 end
			-- fill in listbox values
			local imageSrc=frameEval(frameSet.ImageString,nil,startFrame,endFrame)
			local width,height,imageWidth,imageHeight,text,font,textWidth,textHeight
			if sprite.FixedSize then
				width=tonumber(sprite.Width)
				height=tonumber(sprite.Height)
			else
				width=tonumber(frameEval(frameSet.WidthString,nil,startFrame,endFrame))
				height=tonumber(frameEval(frameSet.HeightString,nil,startFrame,endFrame))
			end
			if type(imageSrc)=="string" then
				local tmpPath=animationMaint.TargetContent.Path
				if string.sub(tmpPath,-1)~="/" then tmpPath=tmpPath.."/" end
				imageSrc=tmpPath..imageSrc
			end
			local tmpImage=Turbine.UI.Control()
			local success,resutl=pcall(Turbine.UI.Control.SetBackground,tmpImage,imageSrc)
			if success then
				tmpImage:SetStretchMode(2)
				imageWidth,imageHeight=tmpImage:GetSize()
			end
			text=""
			if imageSrc==nil or imageSrc=="" then
				text=frameEval(frameSet.TextString,nil,startFrame,endFrame)
				if text==nil then text="nil" end
				font=frameSet.Font
				if font==nil then font=Settings.Font end
				if frameSet.TextString~=nil and frameSet.TextString~="" and (width==nil or height==nil) then
					fontMetric:SetFont(font)
					if width==nil then
						width=fontMetric:GetTextWidth(text)
						if width>displayWidth then width=displayWidth end
					end
					if height==nil then
						height=fontMetric:GetTextHeight(text,width)
					end
				end
			else
				if width==nil then width=imageWidth end
				if height==nil then height=imageHeight end
			end

			for k=1,animationMaint.CurrFrameList:GetItemCount() do
				local item=animationMaint.CurrFrameList:GetItem(k)
				tmpImage=nil
				if item.Type=="color" then
					if item.Name=="BackColor" and sprite~=nil then
						value=frameSet.BackColor
					elseif item.Name=="ForeColor" and sprite~=nil then
						value=frameSet.ForeColor
					end
					if value==nil then
						item.Value:SetVisible(false)
					else
						item.Value:SetBackColor(Turbine.UI.Color(value.R,value.G,value.B))
						item.Value:SetVisible(true)
					end
				elseif item.Type=="label" or item.Type=="text" then
					local value
					if item.Name=="Sprite" then
						local name=string.trim(animationMaint.TargetContent.Sprites[animationMaint.CurrentSpriteIndex].Name)
						if name==nil then name=animationMaint.CurrentSpriteIndex end
						value=name
					elseif item.Name=="Frame" then
						value=frame
					elseif item.Name=="Image" and sprite~=nil then
						value=imageSrc
					elseif item.Name=="Width" and sprite~=nil then
						value=tostring(width)
					elseif item.Name=="Height" and sprite~=nil then
						value=tostring(height)
					elseif item.Name=="Left" and sprite~=nil then
						value=frameEval(frameSet.LeftString,nil,startFrame,endFrame)
						if value==nil then value=0 end
					elseif item.Name=="Top" and sprite~=nil then
						value=frameEval(frameSet.TopString,nil,startFrame,endFrame)
						if value==nil then value=0 end
					elseif item.Name=="ZOrder" and sprite~=nil then
						value=frameEval(frameSet.ZOrderString,nil,startFrame,endFrame)
						if value==nil then value=0 end
					elseif item.Name=="TextString" and sprite~=nil then
						value=text
						if value==nil then value="" end
					elseif item.Name=="font" and sprite~=nil then
						value=font
						if value==nil then
							value=""
						else
							value=Turbine.UI.Lotro.FontInfo[value].name.." "..tostring(Turbine.UI.Lotro.FontInfo[value].size)
						end
					end
					item.Value:SetText(tostring(value))
				end
			end
		else
			for k=1,animationMaint.CurrFrameList:GetItemCount() do
				local item=animationMaint.CurrFrameList:GetItem(k)
				if item.Type=="label" or item.Type=="text" then
					item.Value:SetText(tostring(""))
				end
			end
		end
	else
		for k=1,animationMaint.CurrFrameList:GetItemCount() do
			local item=animationMaint.CurrFrameList:GetItem(k)
			if item.Type=="label" or item.Type=="text" then
				item.Value:SetText(tostring(""))
			end
		end
	end
end
animationMaint.RefreshLayout=function()
	local controlHeight=Turbine.UI.Lotro.FontInfo[Settings.Font].size+4
	local buttonFont=Settings.Font
	local borderHeight=displayHeight-2
	local borderWidth=displayWidth-2
	while Turbine.UI.Lotro.FontInfo[buttonFont].size>22 do
		-- make sure button text fits on buttons since they don't size larger than 20 pixels (22 is ok since lotro font sizes are off a bit)
		buttonFont=GetSmallerFont(buttonFont)
	end
	
	fontMetric:SetFont(Settings.Font)
	local hdrLblWidth=fontMetric:GetTextWidth(Resource[language][263]..":")+10
	local tmp=fontMetric:GetTextWidth(Resource[language][264]..":")+10
	if tmp>hdrLblWidth then hdrLblWidth=tmp end
	local tmp=fontMetric:GetTextWidth(Resource[language][244]..":")+10
	if tmp>hdrLblWidth then hdrLblWidth=tmp end
	local tmp=fontMetric:GetTextWidth(Resource[language][265]..":")+10
	if tmp>hdrLblWidth then hdrLblWidth=tmp end
	local tmp=fontMetric:GetTextWidth(Resource[language][266]..":")+10
	if tmp>hdrLblWidth then hdrLblWidth=tmp end
	local tmp=fontMetric:GetTextWidth(Resource[language][267]..":")+10
	if tmp>hdrLblWidth then hdrLblWidth=tmp end
	local tmp=fontMetric:GetTextWidth(Resource[language][245]..":")+10
	if tmp>hdrLblWidth then hdrLblWidth=tmp end

	animationMaint.Border:SetSize(displayWidth-2,displayHeight-2)
	animationMaint.HeaderPanel:SetSize(displayWidth,controlHeight*3+20)
	animationMaint.Title:SetSize(animationMaint.HeaderPanel:GetWidth(),controlHeight)
	animationMaint.Title:SetPosition(0,5)
	animationMaint.Title:SetFont(Settings.Font)
	animationMaint.Title:SetText(Resource[language][242])

	local hdrColWidth=animationMaint.HeaderPanel:GetWidth()/4
	local hdrFldWidth=hdrColWidth-hdrLblWidth-15
	local hdrFldWidth2=fontMetric:GetTextWidth("8888")+20
	animationMaint.ImagePathCaption:SetSize(hdrLblWidth,controlHeight)
	animationMaint.ImagePathCaption:SetPosition(5,animationMaint.Title:GetTop()+controlHeight+5)
	animationMaint.ImagePathCaption:SetFont(Settings.Font)
	animationMaint.ImagePathCaption:SetText(Resource[language][265]..":")
	animationMaint.ImagePath:SetSize(hdrFldWidth,controlHeight)
	animationMaint.ImagePath:SetPosition(animationMaint.ImagePathCaption:GetLeft()+animationMaint.ImagePathCaption:GetWidth(),animationMaint.ImagePathCaption:GetTop())
	animationMaint.ImagePath:SetFont(Settings.Font)
	animationMaint.ImagePath:SetText(animationMaint.ImagePath:GetText())
	animationMaint.BackImageCaption:SetSize(hdrLblWidth,controlHeight)
	animationMaint.BackImageCaption:SetPosition(hdrColWidth,animationMaint.ImagePathCaption:GetTop())
	animationMaint.BackImageCaption:SetFont(Settings.Font)
	animationMaint.BackImageCaption:SetText(Resource[language][266]..":")
	animationMaint.BackImage:SetSize(hdrFldWidth,controlHeight)
	animationMaint.BackImage:SetPosition(animationMaint.BackImageCaption:GetLeft()+animationMaint.BackImageCaption:GetWidth(),animationMaint.BackImageCaption:GetTop())
	animationMaint.BackImage:SetFont(Settings.Font)
	animationMaint.BackImage:SetText(animationMaint.BackImage:GetText())
	animationMaint.BackColorCaption:SetSize(hdrLblWidth,controlHeight)
	animationMaint.BackColorCaption:SetPosition(hdrColWidth*2,animationMaint.ImagePathCaption:GetTop())
	animationMaint.BackColorCaption:SetFont(Settings.Font)
	animationMaint.BackColorCaption:SetText(Resource[language][267]..":")
	animationMaint.UseBackColor:SetSize(20,controlHeight)
	animationMaint.UseBackColor:SetText(" ")
	animationMaint.UseBackColor:SetPosition(animationMaint.BackColorCaption:GetLeft()+animationMaint.BackColorCaption:GetWidth(),animationMaint.BackColorCaption:GetTop())
	animationMaint.BackColor:SetSize(hdrFldWidth-25,controlHeight)
	animationMaint.BackColor:SetPosition(animationMaint.UseBackColor:GetLeft()+25,animationMaint.BackColorCaption:GetTop())
	animationMaint.WidthCaption:SetSize(hdrLblWidth,controlHeight)
	animationMaint.WidthCaption:SetPosition(5,animationMaint.ImagePathCaption:GetTop()+controlHeight+5)
	animationMaint.WidthCaption:SetFont(Settings.Font)
	animationMaint.WidthCaption:SetText(Resource[language][263]..":")
	animationMaint.Width:SetSize(hdrFldWidth2,controlHeight)
	animationMaint.Width:SetPosition(animationMaint.WidthCaption:GetLeft()+animationMaint.WidthCaption:GetWidth(),animationMaint.WidthCaption:GetTop())
	animationMaint.Width:SetFont(Settings.Font)
	animationMaint.Width:SetText(animationMaint.Width:GetText())
	animationMaint.HeightCaption:SetSize(hdrLblWidth,controlHeight)
	animationMaint.HeightCaption:SetPosition(hdrColWidth,animationMaint.Width:GetTop())
	animationMaint.HeightCaption:SetFont(Settings.Font)
	animationMaint.HeightCaption:SetText(Resource[language][264]..":")
	animationMaint.Height:SetSize(hdrFldWidth2,controlHeight)
	animationMaint.Height:SetPosition(animationMaint.HeightCaption:GetLeft()+animationMaint.HeightCaption:GetWidth(),animationMaint.HeightCaption:GetTop())
	animationMaint.Height:SetFont(Settings.Font)
	animationMaint.Height:SetText(animationMaint.Height:GetText())
	animationMaint.FPSCaption:SetSize(hdrLblWidth,controlHeight)
	animationMaint.FPSCaption:SetPosition(hdrColWidth*2,animationMaint.Width:GetTop())
	animationMaint.FPSCaption:SetFont(Settings.Font)
	animationMaint.FPSCaption:SetText(Resource[language][244]..":")
	animationMaint.FPS:SetSize(hdrFldWidth2,controlHeight)
	animationMaint.FPS:SetPosition(animationMaint.FPSCaption:GetLeft()+animationMaint.FPSCaption:GetWidth(),animationMaint.FPSCaption:GetTop())
	animationMaint.FPS:SetFont(Settings.Font)
	animationMaint.FPS:SetText(animationMaint.FPS:GetText())
	animationMaint.FrameCountCaption:SetSize(hdrLblWidth,controlHeight)
	animationMaint.FrameCountCaption:SetPosition(hdrColWidth*3,animationMaint.Width:GetTop())
	animationMaint.FrameCountCaption:SetFont(Settings.Font)
	animationMaint.FrameCountCaption:SetText(Resource[language][245]..":")
	animationMaint.FrameCount:SetSize(hdrFldWidth2,controlHeight)
	animationMaint.FrameCount:SetPosition(animationMaint.FrameCountCaption:GetLeft()+animationMaint.FrameCountCaption:GetWidth(),animationMaint.FrameCountCaption:GetTop())
	animationMaint.FrameCount:SetFont(Settings.Font)
	animationMaint.FrameCount:SetText(animationMaint.FrameCount:GetText())
	-- frame seperators
	animationMaint.Sep1:SetWidth(animationMaint.Border:GetWidth())
	animationMaint.Sep1:SetPosition(0,AuthorSettings.AnimationMaint.Split1*borderHeight)
	local splitTop=animationMaint.HeaderPanel:GetTop()+animationMaint.HeaderPanel:GetHeight()
	local splitHeight=animationMaint.Sep1:GetTop()-splitTop
	animationMaint.Sep2:SetHeight(splitHeight)
	animationMaint.Sep2:SetPosition(AuthorSettings.AnimationMaint.Split2*borderWidth,splitTop)
	animationMaint.Sep3:SetHeight(splitHeight)
	animationMaint.Sep3:SetPosition(AuthorSettings.AnimationMaint.Split3*borderWidth,splitTop)

	animationMaint.FrameSetPanel:SetPosition(1,animationMaint.HeaderPanel:GetTop()+animationMaint.HeaderPanel:GetHeight()+1)
	animationMaint.FrameSetPanel:SetSize(animationMaint.Sep2:GetLeft()-animationMaint.FrameSetPanel:GetLeft(),splitHeight-1)
	animationMaint.FrameSetList:SetSize(animationMaint.FrameSetPanel:GetWidth()-10,animationMaint.FrameSetPanel:GetHeight())
	animationMaint.FrameSetListVScroll:SetLeft(animationMaint.FrameSetList:GetWidth())
	animationMaint.FrameSetListVScroll:SetHeight(animationMaint.FrameSetList:GetHeight())

	animationMaint.PreviewPanel:SetPosition(animationMaint.Sep2:GetLeft()+animationMaint.Sep2:GetWidth(),animationMaint.FrameSetPanel:GetTop())
	animationMaint.PreviewPanel:SetSize(animationMaint.Sep3:GetLeft()-animationMaint.PreviewPanel:GetLeft(),animationMaint.FrameSetPanel:GetHeight())
	refreshAnimationPreview()

	animationMaint.CurrFramePanel:SetPosition(animationMaint.Sep3:GetLeft()+animationMaint.Sep3:GetWidth(),animationMaint.FrameSetPanel:GetTop())
	animationMaint.CurrFramePanel:SetSize(borderWidth-animationMaint.CurrFramePanel:GetLeft(),splitHeight-1)
	animationMaint.CurrFrameList:SetSize(animationMaint.CurrFramePanel:GetWidth()-10,animationMaint.CurrFramePanel:GetHeight())
	animationMaint.CurrFrameListVScroll:SetLeft(animationMaint.CurrFrameList:GetWidth())
	animationMaint.CurrFrameListVScroll:SetHeight(animationMaint.CurrFrameList:GetHeight())
	animationMaint.FrameScroll:SetTop(animationMaint.Sep1:GetTop()+animationMaint.Sep1:GetHeight())
	animationMaint.FrameScroll:SetWidth(borderWidth)
	animationMaint.TimelinePanel:SetTop(animationMaint.FrameScroll:GetTop()+animationMaint.FrameScroll:GetHeight())
	animationMaint.TimelinePanel:SetSize(borderWidth,borderHeight-animationMaint.TimelinePanel:GetTop()-controlHeight-1)
	animationMaint.TimelineVScroll:SetLeft(animationMaint.TimelinePanel:GetWidth()-10)
	animationMaint.TimelineVScroll:SetHeight(animationMaint.TimelinePanel:GetHeight()-animationMaint.TimelineVScroll:GetTop()-10)
	animationMaint.TimelineHScroll:SetPosition(animationMaint.TimelineHeaderCaption:GetWidth(),animationMaint.TimelinePanel:GetHeight()-10)
	animationMaint.TimelineHScroll:SetWidth(animationMaint.TimelinePanel:GetWidth()-animationMaint.TimelineHScroll:GetLeft()-10)
	animationMaint.TimelineFrameMarker:SetHeight(animationMaint.TimelinePanel:GetHeight()-10)
	refreshTimeline()
	if animationMaint.TargetContent~=nil then
		refreshCurrFrameList(animationMaint.FrameScroll:GetValue()) -- in case scroll max values changed, etc
	end

	animationMaint.Message:SetSize(borderWidth,controlHeight)
	animationMaint.Message:SetTop(animationMaint.TimelinePanel:GetTop()+animationMaint.TimelinePanel:GetHeight()+1)
	animationMaint.Message:SetFont(Settings.Font)
	animationMaint.Message:SetText(animationMaint.Message:GetText())
end
animationMaint.RefreshLayout()
animationMaint.RefreshColor=function()
	animationMaint:SetBackColor(Settings.TrimColor)
	animationMaint.Border:SetBackColor(Settings.TrimColor)
	animationMaint.HeaderPanel:SetBackColor(Settings.BackColor)
	animationMaint.Title:SetForeColor(Settings.HeadingColor)
	animationMaint.WidthCaption:SetForeColor(Settings.LabelColor)
	animationMaint.HeightCaption:SetForeColor(Settings.LabelColor)
	animationMaint.FPSCaption:SetForeColor(Settings.LabelColor)
	animationMaint.ImagePathCaption:SetForeColor(Settings.LabelColor)
	animationMaint.BackImageCaption:SetForeColor(Settings.LabelColor)
	animationMaint.BackColorCaption:SetForeColor(Settings.LabelColor)
	animationMaint.BackColor:SetTrimColor(Settings.TrimColor);
	animationMaint.FrameCountCaption:SetForeColor(Settings.LabelColor)

	animationMaint.Sep1:SetBackColor(Settings.TrimColor)
	animationMaint.Sep2:SetBackColor(Settings.TrimColor)
	animationMaint.Sep3:SetBackColor(Settings.TrimColor)

	animationMaint.FrameSetPanel:SetBackColor(Settings.BackColor)
	animationMaint.PreviewPanel:SetBackColor(Settings.BackColor)
	animationMaint.CurrFramePanel:SetBackColor(Settings.BackColor)
	animationMaint.TimelinePanel:SetBackColor(Settings.BackColor)
	animationMaint.TimelineHeader:SetBackColor(Settings.TrimColor)
	animationMaint.TimelineList:SetBackColor(0.9,Turbine.UI.Color(Settings.TrimColor.R,Settings.TrimColor.G,Settings.TrimColor.B))

	animationMaint.Message:SetForeColor(Settings.LabelColor)
	animationMaint.Message:SetBackColor(Settings.BackColor)
end
animationMaint.RefreshColor()

animationMaint.ShowMaint=function(targetControl)
	animationMaint.CurrentSpriteIndex=nil
	animationMaint.loading=true
	if Settings.DefaultImagePath==nil then Settings.DefaultImagePath="" end
	animationMaint.TargetControl=targetControl
	animationMaint.TargetContent=authorStory.Chapters[chapterMaint.ChapterID].Content[targetControl.Index][authorWindow.CurrLanguage:GetValue()]
	animationMaint.ImagePath:SetText(animationMaint.TargetContent.Path)
	animationMaint.BackImage:SetText(animationMaint.TargetContent.FileName)
	local tmp=animationMaint.TargetContent.BackColor
	if tmp==nil then
		animationMaint.UseBackColor:SetChecked(false)
		animationMaint.BackColor:SetColor(Settings.BackColor)
	else
		animationMaint.UseBackColor:SetChecked(true)
		animationMaint.BackColor:SetColor(Turbine.UI.Color(tmp.R,tmp.G,tmp.B))
	end
	tmp=animationMaint.TargetContent.Width
	if tmp==nil then
		animationMaint.Width:SetText("")
	else
		animationMaint.Width:SetText(tmp)
	end
	tmp=animationMaint.TargetContent.Height
	if tmp==nil then
		animationMaint.Height:SetText("")
	else
		animationMaint.Height:SetText(tmp)
	end
	tmp=animationMaint.TargetContent.FPS
	if tmp==nil then
		animationMaint.FPS:SetText("")
	else
		animationMaint.FPS:SetText(tmp)
	end
	tmp=animationMaint.TargetContent.FrameCount
	if tmp==nil then
		animationMaint.FrameCount:SetText("")
	else
		animationMaint.FrameCount:SetText(tmp)
		tmp=tonumber(tmp)
		if tmp==nil or tmp<1 then tmp=1 end
		animationMaint.FrameScroll:SetMaximum(tmp)
	end
	animationMaint.FrameScroll:SetValue(1)

	-- load timeline header and list
	for k,v in ipairs(animationMaint.TargetContent.Sprites) do
		newSpriteControl(k) -- adds itself to timeline
	end

	refreshTimeline()
	FontSelectList:SetVisible(false)
	initFrameSetList()
	refreshFrameSetList()
	animationMaint.FrameScroll:SetValue(1)
	refreshCurrFrameList(1)
	animationMaint.loading=false
	animationMaint:SetVisible(true)
	refreshAnimationPreview()
end
animationMaint:SetZOrder(3)
