importPath=string.gsub(getfenv(1)._.Name,"%.AnimationViewer","").."."
resourcePath=string.gsub(importPath,"%.","/").."Resources/"

if Settings==nil then Settings={} end
if Settings.TrimColor==nil then Settings.TrimColor=Turbine.UI.Color(.15,.15,.25) end

animationViewer=Turbine.UI.Window()
animationViewer:SetSize(displayWidth,displayHeight)
animationViewer:SetBackColor(Settings.TrimColor)

animationViewer:SetPosition(0,0)

animationViewer:SetZOrder(3) -- chapterMaint is ZOrder 2

animationViewer.ViewPort=Turbine.UI.ListBox()
animationViewer.ViewPort:SetParent(animationViewer)
animationViewer.ViewPort:SetMouseVisible(false)
animationViewer.ViewPort:SetPosition(1,1)
animationViewer.ViewPort:SetSize(displayWidth,displayHeight)
animationViewer.ViewPort:SetBackColor(Turbine.UI.Color.Black)

animationViewer.Canvas=Turbine.UI.Control()
animationViewer.Canvas:SetParent(animationViewer.ViewPort)
animationViewer.Canvas:SetMouseVisible(false)
animationViewer.ViewPort:AddItem(animationViewer.Canvas)

animationViewer.ViewPortVScroll=Turbine.UI.Lotro.ScrollBar()
animationViewer.ViewPortVScroll:SetParent(animationViewer)
animationViewer.ViewPortVScroll:SetOrientation(Turbine.UI.Orientation.Vertical)
animationViewer.ViewPortVScroll:SetSize(10,displayHeight-10)
animationViewer.ViewPortVScroll:SetPosition(displayWidth-10,10)
animationViewer.ViewPort:SetVerticalScrollBar(animationViewer.ViewPortVScroll)

animationViewer.ViewPortHScroll=Turbine.UI.Lotro.ScrollBar()
animationViewer.ViewPortHScroll:SetParent(animationViewer)
animationViewer.ViewPortHScroll:SetOrientation(Turbine.UI.Orientation.Horizontal)
animationViewer.ViewPortHScroll:SetSize(displayWidth-10,10)
animationViewer.ViewPortHScroll:SetPosition(0,displayHeight-10)
animationViewer.ViewPort:SetHorizontalScrollBar(animationViewer.ViewPortHScroll)

animationViewer.ShowMovie=function(moviePath,filePattern,frameCount,targetFPS,width,height,loop)
	local tmpSprites={}
	tmpSprites[1]={}
	tmpSprites[1].Width=width
	tmpSprites[1].Height=height
	tmpSprites[1].FixedSize=true
	tmpSprites[1].FilePattern=filePattern -- custom field for movies
	tmpSprites[1].GetImage=function(self,frame)
		local fileName=string.format(self.FilePattern,frame)
		if animationViewer.AnimationPath=="\\" and tonumber(fileName)~=nil and tonumber(fileName)>0 then
			fileName=tonumber(fileName)
		else
			fileName=animationViewer.AnimationPath..fileName
		end
		return fileName
	end
	animationViewer.ShowAnimation(moviePath,nil,nil,tmpSprites,frameCount,targetFPS,width,height,loop)
end
animationViewer.ShowAnimation=function(animationPath,backImage,backColor,sprites,frameCount,targetFPS,width,height,loop)
	animationViewer.ControlPanel:SetVisible(false)
	-- sprites is a table of sprite definitions
	-- sprites[index].Width= optional width of sprite
	-- sprites[index].Height= optional height of sprite
	-- sprites[index].Left= optional initial left position of sprite
	-- sprites[index].Top= optional initial top position of sprite
	-- sprites[index].ZOrder= optional initial zorder of sprite
	-- sprites[index]:GetImage(frame) is a required method that returns the image name of the sprite (either file name or resource #) for the specified frame
	-- sprites[index]:GetFrame(frame) is a required method that returns the Left, Top, ZOrder, Text, Font, BackColor and ForeColor for the specified frame
	-- note, GetImage and GetFrame are seperate functions because the image is usually loaded one frame ahead to allow pre-loading it as a "Turbine.UI.Graphic" object

	-- viewer calculates the actual width and height here, centering the view or adding scrollbars as needed
	if targetFPS==nil or tonumber(targetFPS)==nil then targetFPS=30 end
	targetFPS=tonumber(targetFPS)
	if targetFPS<1 then targetFPS=30 end
	if loop==nil then loop=false end
	animationViewer.Loop=loop
	animationViewer.FPS=targetFPS
	if frameCount==nil then frameCount=0 end
	animationViewer.FrameCount=frameCount
	local canvasWidth=width
	local canvasHeight=height
	if animationPath==nil then animationPath="" end
	if string.sub(animationPath,-1)~="/" then
		animationPath=animationPath.."/"
	end
	animationViewer.AnimationPath=animationPath
	local tmpImage=Turbine.UI.Control()
	if backImage~=nil and backImage~="" then
		if tonumber(backImage)~=nil and tonumber(backImage)>0 then
			animationViewer.Background=tonumber(backImage)
		else
			animationViewer.Background=animationViewer.AnimationPath..backImage
		end
		local success,result=pcall(Turbine.UI.Control.SetBackground,tmpImage,animationViewer.Background)
		if success then
			tmpImage:SetStretchMode(2)
			canvasWidth,canvasHeight=tmpImage:GetSize()
		else
			animationViewer.Background=nil
		end
	else
		animationViewer.Background=nil
	end
	if animationViewer.Sprites~=nil then
		-- clear out existing images
		for index,sprite in pairs(animationViewer.Sprites) do
			for index,sprite in pairs(animationViewer.Sprites) do
				if sprite.CurrentImage~=nil then
					-- release the previously displayed sprite graphic
					sprite.CurrentImage:SetVisible(false)
					sprite.CurrentImage:SetBackground(nil)
					sprite.CurrentImage:SetParent(nil)
					sprite.CurrentImage=nil
				end
				sprite.NextImage=nil -- release the preloaded graphic
			end
		end
	end
	animationViewer.Sprites=sprites
	if sprites~=nil and type(sprites)=="table" and frameCount>0 then
		local spriteCount=#sprites
		for index,sprite in ipairs(animationViewer.Sprites) do
			-- if GetImage doesn't exist, build it from sprite.FrameSets
			-- Sprites[].FrameSets[].StartFrame = frame where this frameset is placed in author
			-- Sprites[].FrameSets[].NextFrame = startframe of NEXT frameset used for case statement in viewer
			-- Sprites[].FrameSets[].ImageString = equation for the image for this frameset, if nil, then GetImage(frame) will return nil
			-- Sprites[].FrameSets[].WidthString = optional equation for the image width
			-- Sprites[].FrameSets[].HeightString = optional equation for the image height
			-- Sprites[].FrameSets[].LeftString = equation for the sprite Left position for this frameset
			-- Sprites[].FrameSets[].TopString = equation for the sprite Top position for this frameset
			-- Sprites[].FrameSets[].ZOrderString = equation for the sprite ZOrder for this frameset

			if sprite.GetImage==nil then
				if (sprite.FrameSets==nil or type(sprite.FrameSets)~="table" or #sprite.FrameSets==0) then
					sprite.GetImage=function(self,frame)
						local fileName=self.FileName
						if animationViewer.AnimationPath=="\\" and tonumber(fileName)~=nil and tonumber(fileName)>0 then
							fileName=tonumber(fileName)
						else
							fileName=animationViewer.AnimationPath..fileName
						end
						return fileName
					end
				else
					-- compile the frameset definitions into a function
					local funcdef="return function(self,frame)\nlocal image\nlocal startFrame,endFrame=1,1\n"
					funcdef=funcdef.."if frame<1 then\n"
					-- init section (frame=0)
					for i,frameSet in ipairs(sprite.FrameSets) do
						-- for each keyframe, add a section
						if frameSet.NextFrame==nil then
							funcdef=funcdef.."else\n" -- last frameset has no NextFrame
						else
							funcdef=funcdef.."elseif frame<"..tostring(frameSet.NextFrame).." then\n"
						end
						funcdef=funcdef.."startFrame="..tostring(frameSet.StartFrame).."\n"
						if frameSet.NextFrame==nil then
							funcdef=funcdef.."endFrame="..animationViewer.FrameCount.."\n"
						else
							funcdef=funcdef.."endFrame="..tostring(frameSet.NextFrame-1).."\n"
						end
						if frameSet.ImageString~=nil and frameSet.ImageString~="" then
							if tonumber(frameSet.ImageString)~=nil and tonumber(frameSet.ImageString)>0 then
								funcdef=funcdef.."image=tonumber("..frameSet.ImageString..")\n"
							else
								funcdef=funcdef.."image=animationViewer.AnimationPath.."..frameSet.ImageString.."\n"
							end
						else
							funcdef=funcdef.."image=nil\n"
						end
					end
					funcdef=funcdef.."end\nreturn image\nend"
					local tmpFunc,error=loadstring(funcdef)
					if error==nil then
						setfenv(tmpFunc,getfenv()); -- we set the environment to the current plugin environment

						sprite.GetImage=tmpFunc()
					else
						Turbine.Shell.WriteLine(error)
					end
				end
			end
			if sprite.GetFrame==nil then
				if sprite.FrameSets~=nil and type(sprite.FrameSets)=="table" and #sprite.FrameSets>0 then
					local funcdef="return function(self,frame)\nlocal width,height,left,top,zorder,text,font,backcolor,forecolor\nlocal startFrame,endFrame=1,1\n"
					funcdef=funcdef.."if frame<1 then\n"
					-- init section (frame=0)
					for i,frameSet in ipairs(sprite.FrameSets) do
						-- for each keyframe, add a section
						if frameSet.NextFrame==nil then
							funcdef=funcdef.."else\n" -- last frameset has no NextFrame
						else
							funcdef=funcdef.."elseif frame<"..tostring(frameSet.NextFrame).." then\n"
						end
						funcdef=funcdef.."startFrame="..tostring(frameSet.StartFrame).."\n"
						if frameSet.NextFrame==nil then
							funcdef=funcdef.."endFrame="..animationViewer.FrameCount.."\n"
						else
							funcdef=funcdef.."endFrame="..tostring(frameSet.NextFrame-1).."\n"
						end
						if frameSet.WidthString~=nil and frameSet.WidthString~="" then
							funcdef=funcdef.."width="..frameSet.WidthString.."\n"
						end
						if frameSet.HeightString~=nil and frameSet.HeightString~="" then
							funcdef=funcdef.."height="..frameSet.HeightString.."\n"
						end
						if frameSet.LeftString~=nil and frameSet.LeftString~="" then
							funcdef=funcdef.."left="..frameSet.LeftString.."\n"
						end
						if frameSet.TopString~=nil and frameSet.TopString~="" then
							funcdef=funcdef.."top="..frameSet.TopString.."\n"
						end
						if frameSet.ZOrderString~=nil and frameSet.ZOrderString~="" then
							funcdef=funcdef.." zorder="..frameSet.ZOrderString.."\n"
						end
						if frameSet.TextString~=nil and frameSet.TextString~="" then
							funcdef=funcdef.." text="..frameSet.TextString.."\n"
						end
						if frameSet.Font~=nil and frameSet.Font~="" then
							funcdef=funcdef.." font="..frameSet.Font.."\n"
						end
						if frameSet.BackColor~=nil then
							funcdef=funcdef.." backcolor=Turbine.UI.Color("..frameSet.BackColor.R..","..frameSet.BackColor.G..","..frameSet.BackColor.B..")\n"
						end
						if frameSet.ForeColor~=nil then
							funcdef=funcdef.." forecolor=Turbine.UI.Color("..frameSet.ForeColor.R..","..frameSet.ForeColor.G..","..frameSet.ForeColor.B..")\n"
						end
					end
					funcdef=funcdef.."end\nreturn width,height,left,top,zorder,text,font,backcolor,forecolor\nend"
					local tmpFunc,error=loadstring(funcdef)
					if error==nil then
						setfenv(tmpFunc,getfenv()); -- we set the environment to the current plugin environment

						sprite.GetFrame=tmpFunc()
					else
						Turbine.Shell.WriteLine(error)
					end
				end
			end

			-- preload the first image of each sprite
			sprite.CurrentImage=Turbine.UI.Label() -- use labels since we allow image or text controls
			sprite.CurrentImage:SetPosition(0,0)
			sprite.CurrentImage:SetVisible(false)
			sprite.CurrentImage:SetParent(animationViewer.Canvas)
			sprite.CurrentImage:SetMouseVisible(false)
			sprite.CurrentImage:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
			sprite.CurrentImage:SetBlendMode(Turbine.UI.BlendMode.Overlay)
			if sprite.Left~=nil then sprite.CurrentImage:SetLeft(sprite.Left) end
			if sprite.Top~=nil then sprite.CurrentImage:SetTop(sprite.Top) end
			if sprite.ZOrder~=nil then sprite.CurrentImage:SetZOrder(ZOrder) end
			sprite.ShowFrame=function(self,frame)
				local width,height,text,font
				if self.FixedSize then
					width=self.Width
					height=self.Height
				end
				local visible=false
				if frame>0 then
					if self.LastFrame==nil or self.LastFrame+1~=frame and self.GetImage~=nil then
						-- cache is no good, load frame directly
						local success,tmpImage=pcall(Turbine.UI.Graphic,self:GetImage(frame))
						if success then
							self.Cache=tmpImage
						else
							self.Cache=nil
						end
					end
					if self.Cache==nil then
						self.CurrentImage:SetBackColorBlendMode(Turbine.UI.BlendMode.Normal)
					else
						self.CurrentImage:SetBackColorBlendMode(Turbine.UI.BlendMode.Color)
					end
					self.CurrentImage:SetBackground(self.Cache)
					visible=(self.Cache~=nil)
				end
				if self.GetFrame~=nil then
					local tmpWidth,tmpHeight,left,top,zorder,backcolor,forecolor
					if frame==0 then					
						tmpWidth,tmpHeight,left,top,zorder,text,font,backcolor,forecolor=self:GetFrame(1)
					else
						tmpWidth,tmpHeight,left,top,zorder,text,font,backcolor,forecolor=self:GetFrame(frame)
					end
					if not self.FixedSize then
						width=tmpWidth
						height=tmpHeight
					end
					if left==nil then left=self.Left end
					if top==nil then top=self.Top end
					if zorder==nil then zorder=self.ZOrder end
					if left~=nil and left~=self.Left then
						self.CurrentImage:SetLeft(left)
						self.Left=left
					end
					if top~=nil and top~=self.Top then
						self.CurrentImage:SetTop(top)
						self.Top=top
					end
					if zorder~=self.ZOrder then
						self.CurrentImage:SetZOrder(zorder)
						self.ZOrder=zorder
					end
					self.CurrentImage:SetBackColor(backcolor) -- SetBackColor accepts nil
					if forecolor==nil then
						self.CurrentImage:SetForeColor(Settings.FontColor) -- SetForeColor does NOT accept nil
					else
						self.CurrentImage:SetForeColor(forecolor)
					end
				end
				font=tonumber(font)
				if font==nil then font=Settings.Font end
				self.CurrentImage:SetFont(font)
				if text~=nil and text~="" then
					if resolveText~=nil then
						text=resolveText(text)
					end
					self.CurrentImage:SetText(text)
					visible=true
				else
					self.CurrentImage:SetText("")
				end

				if (width==nil or height==nil) then
					local dynSize=false
					if self.Cache~=nil and (width==nil or height==nil) then
						dynSize=true
						local tmpControl=Turbine.UI.Control()
						local success,result=pcall(Turbine.UI.Control.SetBackground,tmpControl,self.Cache)
						if success then
							tmpControl:SetStretchMode(2)
							if width==nil then width=tmpControl:GetWidth() end
							if height==nil then height=tmpControl:GetHeight() end
						end
					end
					if text~=nil and text~="" and (dynSize or width==nil or height==nil)then
						fontMetric:SetFont(font)
						if width==nil or dynSize then
							width=fontMetric:GetTextWidth(text)
							if width>displayWidth then width=displayWidth end
						end
						if height==nil or dynSize then
							height=fontMetric:GetTextHeight(text,width)
							if height>displayHeight then height=displayHeight end
						end
					end
				end
				if self.CurrentImage:GetZOrder()<0 then visible=false end -- negative ZOrder means sprite is behind background
				self.CurrentImage:SetSize(width,height)
				self.CurrentImage:SetVisible(visible)
				if self.GetImage~=nil then
					local tmpSource=self:GetImage(frame+1)
					if tmpSource~=self.LastSource then
						local success,tmpImage=pcall(Turbine.UI.Graphic,tmpSource)
						if success then
							self.LastSource=tmpSource
							self.Cache=tmpImage
						end
					end
				else
					self.Cache=nil
				end
				self.LastFrame=frame
			end
			sprite:ShowFrame(0) -- allow sprite to preload first image and initial position
		end
	else
		animationViewer.Sprites={}
	end

	if canvasWidth==nil then canvasWidth=displayWidth end
	if canvasHeight==nil then canvasHeight=displayHeight end
	if width==nil then width=canvasWidth end
	if height==nil then height=canvasHeight end
	if width>displayWidth then width=displayWidth end
	if height>displayHeight then height=displayHeight end

	-- we are now ready to set up the viewer
	-- then load the image
	animationViewer.Canvas:SetBackground(animationViewer.Background)
	if backColor~=nil then
		animationViewer.Canvas:SetBackColor(backColor)
		if animationViewer.Background~=nil then
			animationViewer.Canvas:SetBackColorBlendMode(Turbine.UI.BlendMode.Color)
		else
			animationViewer.Canvas:SetBackColorBlendMode(Turbine.UI.BlendMode.Normal)
		end
	else
		animationViewer.Canvas:SetBackColor(nil)
		animationViewer.Canvas:SetBackColorBlendMode(Turbine.UI.BlendMode.None)
	end
	if canvasWidth< width-1 then
		-- no scrollbars, center viewer
		animationViewer.Canvas:SetWidth(canvasWidth)
		animationViewer.Canvas:SetLeft((width-canvasWidth)/2)
		animationViewer.ViewPort:SetWidth(width)
		animationViewer:SetWidth(width)
		animationViewer:SetLeft((displayWidth-width)/2-1)
		animationViewer.ViewPort:SetLeft(1)
	elseif canvasWidth<=width then
		-- no scrollbars, center viewer
		animationViewer.Canvas:SetWidth(canvasWidth)
		animationViewer.Canvas:SetLeft((width-canvasWidth)/2)
		animationViewer.ViewPort:SetWidth(width)
		animationViewer:SetWidth(width)
		animationViewer:SetLeft((displayWidth-width)/2)
		animationViewer.ViewPort:SetLeft(0)
	else
		-- need scrollbars, set canvas 10 pixels wider
		animationViewer.ViewPort:SetWidth(width)
		animationViewer.Canvas:SetWidth(canvasWidth+10)
		animationViewer:SetWidth(width)
		animationViewer:SetLeft((displayWidth-width)/2)
		animationViewer.ViewPort:SetLeft(0)
	end
	animationViewer.CloseButton:SetLeft(animationViewer:GetWidth()-16)
	if canvasHeight<= height-1 then
		-- no scrollbars, center image
		animationViewer.ViewPort:SetHeight(height)
		animationViewer.Canvas:SetHeight(canvasHeight)
		animationViewer.Canvas:SetTop((height-canvasHeight)/2)
		animationViewer:SetHeight(height+2)
		animationViewer:SetTop((displayHeight-height)/2-1)
		animationViewer.ViewPort:SetTop(1)
	elseif canvasHeight<= height then
		-- no scrollbars, center image
		animationViewer.ViewPort:SetHeight(height)
		animationViewer.Canvas:SetHeight(canvasHeight)
		animationViewer.Canvas:SetTop((height-canvasHeight)/2)
		animationViewer:SetHeight(height)
		animationViewer:SetTop((displayHeight-height)/2)
		animationViewer.ViewPort:SetTop(0)
	else
		-- need scrollbars, set canvas 10 pixels taller
		animationViewer.ViewPort:SetHeight(height)
		animationViewer.Canvas:SetHeight(canvasHeight+10)
		animationViewer:SetHeight(height)
		animationViewer:SetTop((displayHeight-height)/2)
		animationViewer.ViewPort:SetTop(0)
	end
	animationViewer:SizeChanged()
	animationViewer.ViewPortVScroll:SetPosition(width-10,0)
	animationViewer.ViewPortVScroll:SetHeight(height-10)
	animationViewer.ViewPortHScroll:SetPosition(0,height-10)
	animationViewer.ViewPortHScroll:SetWidth(width-10)
	-- start Update
	animationViewer.Frame=1
	animationViewer.FrameTimeOut=Turbine.Engine:GetGameTime()+1/animationViewer.FPS
	animationViewer.PlayPosition:SetMinimum(1)
	animationViewer.PlayPosition:SetValue(1)
	animationViewer.PlayPosition:SetMaximum(animationViewer.FrameCount)
	-- force a garbage collection
	collectgarbage()
	animationViewer:SetWantsUpdates(true)
	animationViewer:SetVisible(true)
end
animationViewer.CloseButton=Turbine.UI.Control()
animationViewer.CloseButton:SetParent(animationViewer)
animationViewer.CloseButton:SetZOrder(1)
animationViewer.CloseButton:SetSize(16,16)
animationViewer.CloseButton:SetPosition(displayWidth-16,0)
animationViewer.CloseButton:SetBlendMode(Turbine.UI.BlendMode.Overlay)

animationViewer.ControlPanel=Turbine.UI.Control()
animationViewer.ControlPanel:SetParent(animationViewer)
animationViewer.ControlPanel:SetVisible(false)
animationViewer.ControlPanel:SetZOrder(1) -- quarantee that the controlPanel is in front of other controls
animationViewer.ControlPanel:SetMouseVisible(false) -- the panel itself should not intercept mouse clicks, only it's child controls should
animationViewer.ControlPanel:SetHeight(47)
animationViewer.PlayPosition=Turbine.UI.Lotro.ScrollBar()
animationViewer.PlayPosition:SetParent(animationViewer.ControlPanel)
animationViewer.PlayPosition:SetHeight(10)
animationViewer.PlayPosition:SetPosition(10,0)
animationViewer.PlayPosition:SetOrientation(Turbine.UI.Orientation.Horizontal)
animationViewer.PlayPosition:SetBackColor(Turbine.UI.Color(.07,.14,.9))
animationViewer.PlayPosition.ValueChanged=function()
	if not animationViewer.Updating then
		animationViewer.ShowFrame(animationViewer.PlayPosition:GetValue())
	end
end
animationViewer.Rewind=Turbine.UI.Control()
animationViewer.Rewind:SetParent(animationViewer.ControlPanel)
animationViewer.Rewind:SetSize(32,32)
animationViewer.Rewind:SetTop(15)
animationViewer.Rewind:SetBlendMode(Turbine.UI.BlendMode.Overlay)
animationViewer.Rewind:SetBackground(resourcePath.."vcr_rewind.tga")
animationViewer.Rewind.MouseClick=function()
	if not animationViewer.Updating then
		local newFrame=animationViewer.Frame-animationViewer.FPS*10
		if newFrame<1 then newFrame=1 end
		animationViewer.ShowFrame(newFrame)
	end
end
animationViewer.Play=Turbine.UI.Control()
animationViewer.Play:SetParent(animationViewer.ControlPanel)
animationViewer.Play:SetSize(32,32)
animationViewer.Play:SetTop(15)
animationViewer.Play:SetBackground(resourcePath.."vcr_pause.tga")
animationViewer.Play:SetBlendMode(Turbine.UI.BlendMode.Overlay)
animationViewer.Play.MouseClick=function()
	-- pause/resume
	local paused=animationViewer:GetWantsUpdates()
	if paused then
		animationViewer.Play:SetBackground(resourcePath.."vcr_play.tga")
	else
		animationViewer.Play:SetBackground(resourcePath.."vcr_pause.tga")
		if animationViewer.Frame>=animationViewer.FrameCount then
			animationViewer.Frame=1
		end
		animationViewer.FrameTimeOut=Turbine.Engine:GetGameTime()
	end
	animationViewer:SetWantsUpdates(not paused)
end
animationViewer.ShowFrame=function(newFrame)
	if newFrame==nil then newFrame=1 end
	if newFrame>animationViewer.FrameCount then newFrame=animationViewer.FrameCount end
	if newFrame<1 then newFrame=1 end
	for k,v in ipairs(animationViewer.Sprites) do
		v:ShowFrame(newFrame,true)
	end
	animationViewer.Frame=newFrame
	animationViewer.Updating=true
	animationViewer.PlayPosition:SetValue(newFrame)
	animationViewer.Updating=false
	animationViewer.FrameTimeOut=Turbine.Engine:GetGameTime()
	animationViewer.Update() -- force one proper update
end

animationViewer.Forward=Turbine.UI.Control()
animationViewer.Forward:SetParent(animationViewer.ControlPanel)
animationViewer.Forward:SetSize(32,32)
animationViewer.Forward:SetTop(15)
animationViewer.Forward:SetBlendMode(Turbine.UI.BlendMode.Overlay)
animationViewer.Forward:SetBackground(resourcePath.."vcr_forward.tga")
animationViewer.Forward.MouseClick=function()
	if not animationViewer.Updating then
		local newFrame=animationViewer.Frame+animationViewer.FPS*30
		animationViewer.ShowFrame(newFrame)
	end
end

animationViewer.MouseClick=function()
	animationViewer.Play.MouseClick()
end
animationViewer.MouseMove=function()
	animationViewer.ControlPanel:SetVisible(true)
end
animationViewer.MouseHover=function()
	animationViewer.ControlPanel:SetVisible(false)
end

animationViewer.CloseButton.MouseClick=function()
	animationViewer:SetVisible(false)
	animationViewer:SetWantsUpdates(false)
end
if localPlayer:GetAlignment()==2 then
	animationViewer.CloseButton:SetBackground(0x41005f41)
else
	animationViewer.CloseButton:SetBackground(0x41000196)
end


animationViewer.Closed=function()
	animationViewer:SetWantsUpdates(false)
end
animationViewer.Update=function()
	animationViewer.Updating=true
	-- use CurrentFrame to clear the prior frame, then set CurrentFrame to NextFrame and display, then set NextFrame to a new frame and preload all of the images for that frame
	local timeNow=Turbine.Engine:GetGameTime()
	local catchUp=0 -- limit "catch up" to 1 second to prevent endless loops
	if timeNow>=animationViewer.FrameTimeOut then
		-- clear the prior frame
		for k,v in ipairs(animationViewer.Sprites) do
			if v.CurrentImage~=nil then
				v.CurrentImage:SetVisible(false)
				v.CurrentImage:SetBackground(nil)
			end
		end
		while timeNow>=animationViewer.FrameTimeOut and catchUp<animationViewer.FPS do
			catchUp=catchUp+1
			-- advance frames until we reach the current time (this allows dropping frames in case the client is running slower than the desired FPS)
			animationViewer.FrameTimeOut=animationViewer.FrameTimeOut+1/animationViewer.FPS
			if animationViewer.Frame<animationViewer.FrameCount then
				animationViewer.Frame=animationViewer.Frame+1
			else
				-- force a garbage collection on each loop, trying to minimize memory usage - the lotro client still caches some stuff we can't clean up but every bit helps
				collectgarbage()
				if animationViewer.Loop then
					animationViewer.Frame=1
				else
					animationViewer.Frame=animationViewer.FrameCount -- prevent any accidental out of bounds
					animationViewer:SetWantsUpdates(false)
					animationViewer.Play:SetBackground(resourcePath.."vcr_play.tga")
				end
			end
		end
		animationViewer.PlayPosition:SetValue(animationViewer.Frame)
		for k,v in ipairs(animationViewer.Sprites) do
			v:ShowFrame(animationViewer.Frame)
		end
	end
	animationViewer.Updating=false
end
animationViewer.SizeChanged=function()
	local width,height=animationViewer:GetSize()
	animationViewer.ControlPanel:SetWidth(width)
	animationViewer.ControlPanel:SetTop(height-60)
	animationViewer.PlayPosition:SetWidth(width-20)
	animationViewer.Rewind:SetLeft(width/2-58)
	animationViewer.Play:SetLeft(width/2-16)
	animationViewer.Forward:SetLeft(width/2+26)
end
