-- this file will automatically migrate any settings from a prior version to current version. this is only necessary when a format change occurs in the way settings are stored
migrateData=function(priorVersion,storyID)
	if storyID==nil or storyID==0 then
		Turbine.Shell.WriteLine("Migrating Anthology Data...")
		-- process any non-story setting changes and StoryList settings
		if priorVersion==nil or tonumber(priorVersion)<0.6 then
			for k,v in pairs(StoryList) do
				if type(v.Name)~="table" then
					v.Name={[1]=v.Name,[2]=v.NameFR,[3]=v.NameDE}
					v.NameFR=nil
					v.NameDE=nil
					v.Desc={[1]=v.Desc,[2]=v.DescFR,[3]=v.DescDE}
					v.DescFR=nil
					v.DescDE=nil
					v.Supported={[1]=v.EN,[2]=v.FR,[3]=v.DE}
					v.EN=nil
					v.FR=nil
					v.DE=nil
				end
			end
		end
	else
		local tmpStory=PatchDataLoad(Turbine.DataScope.Account,pluginName.."Story"..tostring(StoryList[storyID].ID))
		if type(tmpStory.Name)=="table" and tmpStory.Name[language]~=nil then
			Turbine.Shell.WriteLine("Migrating Story("..tostring(storyID).."):"..tostring(tmpStory.Name[language]))
		else
			Turbine.Shell.WriteLine("Migrating Story("..tostring(storyID)..")")
		end
		if tmpStory~=nil then
			local backup=string.gsub(tostring(tmpStory.DataVersion).."_"..tostring(Turbine.Engine:GetLocalTime()),"%.","_")
			-- create a backup just in case
			PatchDataSave(Turbine.DataScope.Account,pluginName.."Story"..tostring(storyID).."_ver_"..backup,tmpStory)
			migrateStory(tmpStory)
			PatchDataSave(Turbine.DataScope.Account,pluginName.."Story"..tostring(storyID),tmpStory)
		end
	end
end

migrateStory=function(story)
	-- sequentially apply each set of updates starting with the first update that matches the story data version
	local storyVersion=story.DataVersion
	-- bug in authorwindow allowed stories created in 0.5 to still have a blank version
	-- it is critical that no story created in 0.5 be subjected to the migration TO 0.5 as that would wipe out values
	if storyVersion==nil then
		for chapterIndex,chapter in ipairs(story.Chapters) do
			for contentIndex,content in ipairs(chapter.Content) do
				if content[1]~=nil and type(content[1])=="table" then
					storyVersion="0.5"
					break
				end
			end
			if storyVersion~=nil then
				break
			end
		end
	end
	if storyVersion==nil or storyVersion=="0.2" or storyVersion=="0.3" or storyVersion=="0.4" then
		-- story.chapters[x].Content is now saved per language
		-- so story.Chapter[x].Content[y].Text is now story.Chapter[x].Content[y][1].Text
		-- and story.Chapters[x].Content[y].FileNameFR is now story.Chapters[x].Content[y][2].FileName
		for chapterIndex,chapter in ipairs(story.Chapters) do
			for contentIndex,content in ipairs(chapter.Content) do
				content[1]={}
				content[2]={}
				content[3]={}
				content[1].Text=content.Text
				content[1].Path=content.Path
				content[1].FileName=content.FileName
				content[1].FilePattern=content.FilePattern
				content[1].FrameCount=content.FrameCount
				content[1].FPS=content.FPS
				content[1].Width=content.Width
				content[1].Height=content.Height
				content[1].Loop=content.Loop
				content[1].IsValid=content.IsValid
				content.Text=nil
				content.Path=nil
				content.FileName=nil
				content.FilePattern=nil
				content.FrameCount=nil
				content.FPS=nil
				content.Width=nil
				content.Height=nil
				content.Loop=nil
				content.IsValid=nil
				content[2].Text=content.TextFR
				content[2].Path=content.PathFR
				content[2].FileName=content.FileNameFR
				content[2].FilePattern=content.FilePatternFR
				content[2].FrameCount=content.FrameCountFR
				content[2].FPS=content.FPSFR
				content[2].Width=content.WidthFR
				content[2].Height=content.HeightFR
				content[2].Loop=content.LoopFR
				content[2].IsValid=content.IsValidFR
				content.FilePatternFR=nil
				content.FrameCountFR=nil
				content.FPSFR=nil
				content.WidthFR=nil
				content.HeightFR=nil
				content.LoopFR=nil
				content.TextFR=nil
				content.PathFR=nil
				content.FileNameFR=nil
				content.IsValidFR=nil
				content[3].Text=content.TextDE
				content[3].Path=content.PathDE
				content[3].FileName=content.FileNameDE
				content[3].FilePattern=content.FilePatternDE
				content[3].FrameCount=content.FrameCountDE
				content[3].FPS=content.FPSDE
				content[3].Width=content.WidthDE
				content[3].Height=content.HeightDE
				content[3].Loop=content.LoopDE
				content[3].IsValid=content.IsValidDE
				content.FilePatternDE=nil
				content.FrameCountDE=nil
				content.FPSDE=nil
				content.WidthDE=nil
				content.HeightDE=nil
				content.LoopDE=nil
				content.TextDE=nil
				content.PathDE=nil
				content.FileNameDE=nil
				content.IsValidDE=nil
			end
		end
		story.DataVersion="0.5"
		storyVersion="0.5"
	end
	if storyVersion=="0.5" then
		story.Name={[1]=story.Name,[2]=story.NameFR,[3]=story.NameDE}
		story.Desc={[1]=story.Desc,[2]=story.DescFR,[3]=story.DescDE}
		story.Supported={[1]=story.EN,[2]=story.FR,[3]=story.DE}
		story.EN=nil
		story.NameFR=nil
		story.DescFR=nil
		story.FR=nil
		story.NameDE=nil
		story.DescDE=nil
		story.DE=nil
		for k,v in ipairs(story.Chapters) do
			v.Name={[1]=v.Name,[2]=v.NameFR,[3]=v.NameDE}
			v.NameFR=nil
			v.NameDE=nil
			for index,obj in ipairs(v.Objectives) do
				obj.Text={[1]=obj.Text,[2]=obj.TextFR,[3]=obj.TextDE}
				obj.ObjectiveString={[1]=obj.ObjectiveString,[2]=obj.ObjectiveStringFR,[3]=obj.ObjectiveStringDE}
				obj.TextFR=nil
				obj.TextDE=nil
				obj.ObjectiveStringFR=nil
				obj.ObjectiveStringDE=nil
			end
		end
		story.DataVersion="1.0"
		storyVersion="1.0"
	end
end